#Copyright ReportLab Europe Ltd. 2000-2017
#see license.txt for license details
__version__='3.3.0'
from tools.docco.rl_doc_utils import *

heading1("Graphics")

heading2("Introduction")

disc("""
ReportLab Graphics is one of the sub-packages to the ReportLab
library. It started off as a stand-alone set of programs, but is now a
fully integrated part of the ReportLab toolkit that allows you to use
its powerful charting and graphics features to improve your PDF forms
and reports.
""")




#copyright ReportLab Europe Limited. 2000-2016
#see license.txt for license details
__version__='3.3.0'
__doc__="""
This contains tests for the encryption algorithms.

The algorithmic approach is to take values from a known
readable-but-secured PDF file and put them in as assertions.
If a platform varies in any way or we change an algorithm
by mistake, it should shriek at us.

It also generates a directory of files to scan by eyeball,
with meaningful names to suggest the properties they have.

"""
import unittest
from reportlab.lib.testutils import setOutDir,makeSuiteForClasses, outputfile, printLocation
setOutDir(__name__)
from reportlab.pdfgen.canvas import Canvas
from reportlab.lib.pdfencrypt import computeO, \
    computeU, hexText, unHexText, encryptionkey, encodePDF, \
    encryptCanvas

VERBOSE = 0

class EncryptionAlgorithmTestCase(unittest.TestCase):
    """Acrobat algorithms.  Two specific cases known to work.

    We are dealing with 8 bit strings which may contain nasty
    escape characters, get trashed in FTP or editors etc.  Therefore
    I am using two explicit routines hexText and unHexText which
    provide a 'safe' way to represent strings as hex and which are
    not going to vary with Python versions"""

    def check0wnerHash40Bit(self):
        "owner key calculation"
        ownerHash = computeO('userpass','ownerpass', revision=2)
        self.assertEqual(hexText(ownerHash),'<F86213EB0CED81F097947F3B343E34CAC8CA92CE8F6FEE2556FA31EC1FE968AF>')
        
    def checkEncryptionKey40Bit(self):
        userPass = 'userpass'
        ownerHash = unHexText('<F86213EB0CED81F097947F3B343E34CAC8CA92CE8F6FEE2556FA31EC1FE968AF>')
        documentID = 'xxxxxxxxxxxxxxxx'        
        permissions = -4
        encKey = encryptionkey(userPass, ownerHash, permissions, documentID, revision=2)
        self.assertEqual(hexText(encKey),'<7EBBD07A88>')

    def checkUserHash40Bit(self):
        encKey = unHexText('<7EBBD07A88>')
        userHash = computeU(encKey, revision=2, documentId='xxxxxxxxxxxxxxxx')
        self.assertEqual(hexText(userHash),'<AA154131D8FA105317F7104D2001A345D78A3DEEFA3D85D032FC9B4B35DA72A0>')

    def checkEncryptString40Bit(self):
        self.assertEqual(hexText(encodePDF(unHexText('<3DC3EBDA71>'), 9, 0, 'anonymous')), '<57AC33DDEB5775982A>')

        
    def check0wnerHash128Bit(self):
        "owner key calculation"
        ownerHash = computeO('userpass','ownerpass', revision=3)
        self.assertEqual(hexText(ownerHash), '<68E5704AC779A5F0CD89704406587A52F25BF61CADC56A0F8DB6C4DB0052534D>')
        
    def checkEncryptionKey128Bit(self):
        userPass = 'userpass'
        ownerHash = unHexText('<68E5704AC779A5F0CD89704406587A52F25BF61CADC56A0F8DB6C4DB0052534D>')
        documentID = 'xxxxxxxxxxxxxxxx'        
        permissions = -4
        encKey = encryptionkey(userPass, ownerHash, permissions, documentID, revision=3)
        self.assertEqual(hexText(encKey), '<13DDE7585D9BE366C976DDD56AF541D1>')
        
    def checkUserHash128Bit(self):
        encKey = unHexText('<13DDE7585D9BE366C976DDD56AF541D1>')
        userHash = computeU(encKey, revision=3, documentId='xxxxxxxxxxxxxxxx')
        self.assertEqual(hexText(userHash), '<A9AE45CDE827FE0B7D6536267948836A00000000000000000000000000000000>')

    def checkEncryptString128Bit(self):
        self.assertEqual(hexText(encodePDF(unHexText('<3C0C5EBE0122D8EB2BDDF8A09FA8E29E>'),
                                 9,
                                 0,
                                 'anonymous')
                       ),'<27FB3E943FCF61878B>')

class EyeballTestCase(unittest.TestCase):
    "This makes a gaxillion self-explanatory files"
    def check40BitOptions(self):
        userPass = 'userpass'
        for canPrint in (0, 1):
            for canModify in (0, 1):
                for canCopy in (0, 1):
                    for canAnnotate in (0, 1):
                        for strength in (40, 128):
                            # work out a 4-char string to be a mnemonic for the options
                            p = m = c = a = 'x'
                            if canPrint: p = 'P'
                            if canModify: m = 'M'
                            if canCopy: c = 'C'
                            if canAnnotate: a = 'A'

                            filename = 'test_crypto_%03dbit_%s_%s%s%s%s.pdf' % (
                                strength, userPass, p, m, c, a)
                            import os
                            filepath = outputfile(filename)
                            canv = Canvas(filepath)
                            
                            canv.setFont('Helvetica', 24)
                            canv.drawString(100,700, 'PDF Encryption test case')
                            canv.setFont('Helvetica', 16)
                            canv.drawString(100, 675, 'Verify by looking at File - Document Info - Security')

                            canv.drawString(100, 600, 'open password = %s' % userPass)
                            canv.drawString(100, 575, 'strength = %d buts' % strength)
                            canv.drawString(100, 500, 'canPrint = %d' % canPrint)
                            canv.drawString(100, 475, 'canModify = %d' % canModify)
                            canv.drawString(100, 450, 'canCopy = %d' % canCopy)
                            canv.drawString(100, 425, 'canAnnotate = %d' % canAnnotate)

                            encryptCanvas(canv,
                                          userPass,
                                          canPrint=canPrint,
                                          canModify=canModify,
                                          canCopy=canCopy,
                                          canAnnotate=canAnnotate,
                                          strength=strength)
                            
                            canv.save()
                            if VERBOSE: print('saved %s' % filepath)
        
def makeSuite():
    return unittest.TestSuite((
        unittest.makeSuite(EncryptionAlgorithmTestCase,'check'),
        unittest.makeSuite(EyeballTestCase,'check'),
        ))
        
if __name__=='__main__':
    unittest.TextTestRunner().run(makeSuite())



#Copyright ReportLab Europe Ltd. 2000-2017
#see license.txt for license details
"""Tests for the reportlab.lib.colors module.
"""
__version__='3.3.0'
from reportlab.lib.testutils import setOutDir,makeSuiteForClasses, outputfile, printLocation
setOutDir(__name__)
import os, math
import unittest
from reportlab.pdfgen.canvas import Canvas
import reportlab.pdfgen.canvas
from reportlab.lib import colors
from reportlab.lib.units import inch, cm

def framePage(canvas, title):
    canvas.setFont('Times-BoldItalic',20)
    canvas.drawString(inch, 10.5 * inch, title)

    canvas.setFont('Times-Roman',10)
    canvas.drawCentredString(4.135 * inch, 0.75 * inch,
                            'Page %d' % canvas.getPageNumber())

    #draw a border
    canvas.setStrokeColorRGB(1,0,0)
    canvas.setLineWidth(5)
    canvas.line(0.8 * inch, inch, 0.8 * inch, 10.75 * inch)
    #reset carefully afterwards
    canvas.setLineWidth(1)
    canvas.setStrokeColorRGB(0,0,0)


class ColorTestCase(unittest.TestCase):
    ""

    def ctAssertRaisesRegex(self,ex,regex,func,*args,**kwds):
        try:
            a = self.assertRaisesRegex
        except AttributeError:
            a = self.assertRaisesRegexp
        return a(ex,regex,func,*args,**kwds)

    def test0(self):
        "Test color2bw function on all named colors."

        cols = list(colors.getAllNamedColors().values())
        for col in cols:
            gray = colors.color2bw(col)
            r, g, b = gray.red, gray.green, gray.blue
            assert r == g == b


    def test1(self):
        "Test colorDistance function."

        cols = list(colors.getAllNamedColors().values())
        for col in cols:
            d = colors.colorDistance(col, col)
            assert d == 0


    def test2(self):
        "Test toColor function on half a dozen ways to say 'red'."

        allRed = [colors.red, [1, 0, 0], (1, 0, 0),
                  b'red', b'RED', b'0xFF0000', b'0xff0000',b'rgb(255,0,0)',
                  u'red', u'RED', u'0xFF0000', u'0xff0000',u'rgb(255,0,0)',
                  ]

        for thing in allRed:
            assert colors.toColor(thing) == colors.red,"colors.toColor(%s)-->%s != colors.red(%s)" % (ascii(thing),ascii(colors.toColor(thing)),colors.red)

    def test2a(self):
        '''attempt to test toColor against simple attacks'''
        ofn = outputfile('dumbo.txt')
        self.assertRaises(ValueError,colors.toColor,"open(%s,'w').write('dumber and dumber')" % repr(ofn))
        self.assertFalse(os.path.isfile(ofn),"toColor managed to create a file %s :("% repr(ofn))
        self.assertRaises(ValueError,colors.toColor,"red.__class__.__bases__[0].__subclasses__()")
        self.assertRaises(ValueError,colors.toColor,
            '''(lambda fc=(lambda n: [c for c in ().__class__.__bases__[0].__subclasses__() if c.__name__ == n][0]): fc("function")(fc("code")(0,0,0,0,"KABOOM",(), (),(),"","",0,""),{})())()''')
        self.assertEqual(colors.Blacker(colors.red,0.5),colors.toColor("Blacker(red,0.5)"))

    def test3(self):
        "Test roundtrip RGB to CMYK conversion."

        # Take all colors as test subjects, except 'transparent'.
##        rgbCols = colors.getAllNamedColors()
##        del rgbCols['transparent']
        rgbCols = list(colors.getAllNamedColors().items())

        # Make a roundtrip test (RGB > CMYK > RGB).
        for name, rgbCol in rgbCols:
            r1, g1, b1 = rgbCol.red, rgbCol.green, rgbCol.blue
            c, m, y, k = colors.rgb2cmyk(r1, g1, b1)
            r2, g2, b2 = colors.cmyk2rgb((c, m, y, k))
            rgbCol2 = colors.Color(r2, g2, b2)

            # Make sure the differences for each RGB component
            # isreally small (< power(10, -N)!
            N = 16 # max. value found to work on Python2.0 and Win2K.
            deltas = list(map(abs, (r1-r2, g1-g2, b1-b2)))
            assert deltas < [math.pow(10, -N)] * 3

    def test4(self):
        "Construct CMYK instances and test round trip conversion"

        rgbCols = list(colors.getAllNamedColors().items())

        # Make a roundtrip test (RGB > CMYK > RGB).
        for name, rgbCol in rgbCols:
            r1, g1, b1 = rgbCol.red, rgbCol.green, rgbCol.blue
            c, m, y, k = colors.rgb2cmyk(r1, g1, b1)
            cmykCol = colors.CMYKColor(c,m,y,k)
            r2, g2, b2 = cmykCol.red, cmykCol.green, cmykCol.blue #colors.cmyk2rgb((c, m, y, k))
            rgbCol2 = colors.Color(r2, g2, b2)

            # Make sure the differences for each RGB component
            # isreally small (< power(10, -N)!
            N = 16 # max. value found to work on Python2.0 and Win2K.
            deltas = list(map(abs, (r1-r2, g1-g2, b1-b2)))
            assert deltas < [math.pow(10, -N)] * 3

    def test5(self):
        "List and display all named colors and their gray equivalents."

        canvas = reportlab.pdfgen.canvas.Canvas(outputfile('test_lib_colors.pdf'))

        #do all named colors
        framePage(canvas, 'Color Demo - page %d' % canvas.getPageNumber())

        all_colors = list(reportlab.lib.colors.getAllNamedColors().items())
        all_colors.sort() # alpha order by name
        canvas.setFont('Times-Roman', 10)
        text = 'This shows all the named colors in the HTML standard (plus their gray and CMYK equivalents).'
        canvas.drawString(72,740, text)

        canvas.drawString(200,725,'Pure RGB')
        canvas.drawString(300,725,'B&W Approx')
        canvas.drawString(400,725,'CMYK Approx')

        y = 700
        for (name, color) in all_colors:
            canvas.setFillColor(colors.black)
            canvas.drawString(100, y, name)
            canvas.setFillColor(color)
            canvas.rect(200, y-10, 80, 30, fill=1)
            canvas.setFillColor(colors.color2bw(color))
            canvas.rect(300, y-10, 80, 30, fill=1)

            c, m, yel, k = colors.rgb2cmyk(color.red, color.green, color.blue)
            CMYK = colors.CMYKColor(c,m,yel,k)
            canvas.setFillColor(CMYK)
            canvas.rect(400, y-10, 80, 30, fill=1)

            y = y - 40
            if y < 100:
                canvas.showPage()
                framePage(canvas, 'Color Demo - page %d' % canvas.getPageNumber())
                canvas.setFont('Times-Roman', 10)
                y = 700
                canvas.drawString(200,725,'Pure RGB')
                canvas.drawString(300,725,'B&W Approx')
                canvas.drawString(400,725,'CMYK Approx')

        canvas.save()

    def test6(self):
        '''test HexColor'''
        HexColor = colors.HexColor
        Color = colors.Color
        self.assertEqual(HexColor(0xffffff),Color(1,1,1,1))
        self.assertEqual(HexColor(16777215),Color(1,1,1,1))
        self.assertEqual(HexColor(b'#ffffff'),Color(1,1,1,1))
        self.assertEqual(HexColor(b'#FFFFFF'),Color(1,1,1,1))
        self.assertEqual(HexColor(b'0xffffff'),Color(1,1,1,1))
        self.assertEqual(HexColor(b'0xFFFFFF'),Color(1,1,1,1))
        self.assertEqual(HexColor(b'16777215'),Color(1,1,1,1))
        self.ctAssertRaisesRegex(ValueError,r"invalid literal for int\(\) with base 10:.*ffffff",HexColor,b'ffffff')
        self.assertEqual(HexColor(b'#FFFFFF', htmlOnly=True),Color(1,1,1,1))
        self.ctAssertRaisesRegex(ValueError,"not a hex string",HexColor,b'0xffffff',htmlOnly=True)
        self.ctAssertRaisesRegex(ValueError,"not a hex string",HexColor,b'16777215',htmlOnly=True)
        self.assertEqual(HexColor(u'#ffffff'),Color(1,1,1,1))
        self.assertEqual(HexColor(u'#FFFFFF'),Color(1,1,1,1))
        self.assertEqual(HexColor(u'0xffffff'),Color(1,1,1,1))
        self.assertEqual(HexColor(u'0xFFFFFF'),Color(1,1,1,1))
        self.assertEqual(HexColor(u'16777215'),Color(1,1,1,1))
        self.ctAssertRaisesRegex(ValueError,r"invalid literal for int\(\) with base 10:.*ffffff",HexColor,u'ffffff')
        self.assertEqual(HexColor(u'#FFFFFF', htmlOnly=True),Color(1,1,1,1))
        self.ctAssertRaisesRegex(ValueError,"not a hex string",HexColor,u'0xffffff',htmlOnly=True)
        self.ctAssertRaisesRegex(ValueError,"not a hex string",HexColor,u'16777215',htmlOnly=True)

def makeSuite():
    return makeSuiteForClasses(ColorTestCase)


#noruntests
if __name__ == "__main__":
    unittest.TextTestRunner().run(makeSuite())
    printLocation()



from reportlab.lib.testutils import setOutDir,makeSuiteForClasses, outputfile, printLocation, NearTestCase
setOutDir(__name__)
import unittest
from reportlab.pdfgen import canvas
from reportlab.pdfbase import pdfform

class PdfFormTestCase(NearTestCase):
    def testMultipleUsage(self):
        for i in range(2):
            c = canvas.Canvas(outputfile('test_pdfbase_pdfform_multiple_usage_%s.pdf'%i))
            c.drawString(100, 100, "Test")
            pdfform.buttonFieldAbsolute(c, 'button', 'Off', 200, 200)
            c.save()

    def testAAbsoluteAndRelativeFields(self):
        #the old test1 in pdfform
        c = canvas.Canvas(outputfile("test_pdfbase_pdfform_formtest.pdf"))
        # first page
        c.setFont("Courier", 10)
        c.drawString(100, 500, "hello world")
        pdfform.textFieldAbsolute(c, "fieldA", 100, 600, 100, 20, "default value")
        pdfform.textFieldAbsolute(c, "fieldB", 100, 300, 100, 50, "another default value", multiline=1)
        pdfform.selectFieldAbsolute(c, "fieldC", "France", ["Canada", "France", "China"], 100, 200, 100, 20)
        c.rect(100, 600, 100, 20)
        pdfform.buttonFieldAbsolute(c, "field2", "Yes", 100, 700, width=20, height=20)
        c.rect(100, 700, 20, 20)
        pdfform.buttonFieldAbsolute(c, "field3", "Off", 100, 800, width=20, height=20)
        c.rect(100, 800, 20, 20)
        # second page
        c.showPage()
        c.setFont("Helvetica", 7)
        c.translate(50, 20)
        c.drawString(100, 500, "hello world")
        pdfform.textFieldRelative(c, "fieldA_1", 100, 600, 100, 20, "default value 2")
        c.setStrokeColorRGB(1,0,0)
        c.setFillColorRGB(0,1,0.5)
        pdfform.textFieldRelative(c, "fieldB_1", 100, 300, 100, 50, "another default value 2", multiline=1)
        pdfform.selectFieldRelative(c, "fieldC_1", "France 1", ["Canada 0", "France 1", "China 2"], 100, 200, 100, 20)
        c.rect(100, 600, 100, 20)
        pdfform.buttonFieldRelative(c, "field2_1", "Yes", 100, 700, width=20, height=20)
        c.rect(100, 700, 20, 20)
        pdfform.buttonFieldRelative(c, "field3_1", "Off", 100, 800, width=20, height=20)
        c.rect(100, 800, 20, 20)
        c.save()

    def testNewAcroform(self):
        from reportlab.lib.colors import toColor, black, red, green, blue, magenta, yellow, pink
        canv = canvas.Canvas(outputfile("test_pdfbase_acroform-0.pdf"))
        doc = canv._doc
        af = canv.acroForm

        #these are absolute in unchanged matrix
        canv.drawString(4*72,800,'unshifted absolute')
        af.checkbox(name='cb1A',tooltip='Field cb1A',checked=True,x=72,y=72,buttonStyle='circle',borderWidth=1, borderColor=red, fillColor=green, textColor=blue,forceBorder=True)
        af.checkbox(name='cb1B',tooltip='Field cb1B',checked=True,x=72,y=72+36,buttonStyle='check',borderWidth=2, borderColor=green, fillColor=blue, textColor=red,forceBorder=True)
        af.checkbox(name='cb1C',tooltip='Field cb1C',checked=True,x=72,y=72+2*36,buttonStyle='cross',borderWidth=2, borderColor=red, fillColor=green, textColor=blue,forceBorder=True)
        af.checkbox(name='cb1D',tooltip='Field cb1D',checked=True,x=72,y=72+3*36,buttonStyle='star',borderWidth=2, borderColor=red, fillColor=green, textColor=blue,forceBorder=True)
        af.checkbox(name='cb1E',tooltip='Field cb1E',checked=True,x=72,y=72+4*36,buttonStyle='diamond',borderStyle='bevelled', borderWidth=2, borderColor=red, fillColor=green, textColor=blue,forceBorder=True)
        af.checkbox(name='cb1F',tooltip='Field cb1F',checked=True,x=72,y=72+5*36,buttonStyle='check', borderWidth=2, borderColor=red, fillColor=None, textColor=None,forceBorder=True)
        af.checkbox(name='cb1H',tooltip='Field cb1H',checked=True,x=72,y=72+6*36,buttonStyle='check', borderStyle='underlined',borderWidth=2, borderColor=red, fillColor=None, textColor=None,forceBorder=True)
        af.checkbox(name='cb1G',tooltip='Field cb1G',checked=True,x=72,y=72+7*36,buttonStyle='check', borderStyle='dashed',borderWidth=2, borderColor=red, fillColor=None, textColor=None,forceBorder=True)
        af.checkbox(name='cb1I',tooltip='Field cb1I',checked=True,x=72,y=72+8*36,buttonStyle='check', borderStyle='inset',borderWidth=1, borderColor=red, fillColor=None, textColor=None,forceBorder=True)
        af.checkbox(name='cb1J',tooltip='Field cb1J',checked=True,x=72,y=72+9*36,buttonStyle='check', borderStyle='solid',shape='circle', borderWidth=2, borderColor=red, fillColor=green, textColor=blue,forceBorder=True)
        af.checkbox(name='cb1K',tooltip='Field cb1K',checked=True,x=72,y=72+10*36,buttonStyle='check', borderWidth=1, borderColor=None, fillColor=None, textColor=None,forceBorder=True)
        af.checkbox(name='cb1L',tooltip='Field cb1L',checked=False,x=72,y=800,buttonStyle='check',borderWidth=None, borderColor=None, fillColor=None, textColor=None,forceBorder=True)
        af.checkbox(name='cb1M',tooltip='Field cb1M',checked=False,x=72,y=600,buttonStyle='check',borderWidth=2, borderColor=blue, fillColor=None, textColor=None,forceBorder=True)
        af.radio(name='rb1A',tooltip='Field rb1A', value='V1', selected=False,x=144,y=72+0*36,buttonStyle='circle', borderStyle='solid',shape='circle', borderWidth=2, borderColor=red, fillColor=green, textColor=blue,forceBorder=True)
        af.radio(name='rb1A',tooltip='Field rb1A', value='V2', selected=True,x=144,y=72+1*36,buttonStyle='circle', borderStyle='solid',shape='circle', borderWidth=2, borderColor=red, fillColor=green, textColor=blue,forceBorder=True)
        af.radio(name='rb1B',tooltip='Field rb1B', value='V1', selected=False,x=144+36,y=72+0*36,buttonStyle='check', borderStyle='solid',shape='square', borderWidth=2, borderColor=green, fillColor=red, textColor=blue,forceBorder=True)
        af.radio(name='rb1B',tooltip='Field rb1B', value='V2', selected=False,x=144+36,y=72+1*36,buttonStyle='check', borderStyle='solid',shape='square', borderWidth=2, borderColor=green, fillColor=red, textColor=blue,forceBorder=True)
        af.radio(name='rb1C',tooltip='Field rb1C', value='V1', selected=False,x=144*2,y=72+0*36,buttonStyle='circle', borderStyle='bevelled',shape='circle', borderWidth=2, borderColor=green, fillColor=yellow, textColor=blue,forceBorder=True)
        af.radio(name='rb1C',tooltip='Field rb1C', value='V2', selected=True,x=144*2,y=72+1*36,buttonStyle='circle', borderStyle='inset',shape='circle', borderWidth=2, borderColor=magenta, fillColor=pink, textColor=blue,forceBorder=True)
        af.textfield(name='tf1A',tooltip='Field tf1A',value='Hello World',x=144*2+36,y=72+0*36, borderStyle='inset', borderWidth=2, borderColor=magenta, fillColor=pink, textColor=blue,forceBorder=True)
        af.textfield(name='tf1B',tooltip='Field tf1B',value='Hello World',x=144*2+36,y=72+2*36, borderStyle='inset', borderWidth=2, fontName='Courier-Bold', borderColor=magenta, fillColor=pink, textColor=blue,forceBorder=True)
        af.textfield(name='tf1C',tooltip='Field tf1C',value='Hello World',x=144*2+36,y=72+3*36, borderStyle='inset', borderWidth=0, fontName='Courier-Bold', borderColor=green, fillColor=red, textColor=black)
        canv.showPage()
        canv.translate(72,0)    #shift matrix
        #these are absolute in changed matrix
        canv.drawString(4*72,800,'shifted absolute')
        af.checkbox(name='cb2A',tooltip='Field cb2A',checked=True,x=72,y=72,buttonStyle='circle',borderWidth=1, borderColor=red, fillColor=green, textColor=blue,forceBorder=True)
        af.checkbox(name='cb2B',tooltip='Field cb2B',checked=True,x=72,y=72+36,buttonStyle='check',borderWidth=2, borderColor=green, fillColor=blue, textColor=red,forceBorder=True)
        af.checkbox(name='cb2C',tooltip='Field cb2C',checked=True,x=72,y=72+2*36,buttonStyle='cross',borderWidth=2, borderColor=red, fillColor=green, textColor=blue,forceBorder=True)
        af.checkbox(name='cb2D',tooltip='Field cb2D',checked=True,x=72,y=72+3*36,buttonStyle='star',borderWidth=2, borderColor=red, fillColor=green, textColor=blue,forceBorder=True)
        af.checkbox(name='cb2E',tooltip='Field cb2E',checked=True,x=72,y=72+4*36,buttonStyle='diamond',borderStyle='bevelled', borderWidth=2, borderColor=red, fillColor=green, textColor=blue,forceBorder=True)
        af.checkbox(name='cb2F',tooltip='Field cb2F',checked=True,x=72,y=72+5*36,buttonStyle='check', borderWidth=2, borderColor=red, fillColor=None, textColor=None,forceBorder=True)
        af.checkbox(name='cb2H',tooltip='Field cb2H',checked=True,x=72,y=72+6*36,buttonStyle='check', borderStyle='underlined',borderWidth=2, borderColor=red, fillColor=None, textColor=None,forceBorder=True)
        af.checkbox(name='cb2G',tooltip='Field cb2G',checked=True,x=72,y=72+7*36,buttonStyle='check', borderStyle='dashed',borderWidth=2, borderColor=red, fillColor=None, textColor=None,forceBorder=True)
        af.checkbox(name='cb2I',tooltip='Field cb2I',checked=True,x=72,y=72+8*36,buttonStyle='check', borderStyle='inset',borderWidth=2, borderColor=red, fillColor=None, textColor=None,forceBorder=True)
        af.checkbox(name='cb2J',tooltip='Field cb2J',checked=True,x=72,y=72+9*36,buttonStyle='check', borderStyle='solid',shape='circle', borderWidth=2, borderColor=red, fillColor=green, textColor=blue,forceBorder=True)
        af.checkbox(name='cb2K',tooltip='Field cb2K',checked=True,x=72,y=72+10*36,buttonStyle='check', borderWidth=1, borderColor=None, fillColor=None, textColor=None,forceBorder=True)
        af.checkbox(name='cb2L',tooltip='Field cb2L',checked=False,x=72,y=800,buttonStyle='check',borderWidth=None, borderColor=None, fillColor=None, textColor=None,forceBorder=True)
        af.checkbox(name='cb2M',tooltip='Field cb2M',checked=False,x=72,y=600,buttonStyle='check',borderWidth=2, borderColor=blue, fillColor=None, textColor=None,forceBorder=True)
        af.radio(name='rb2A',tooltip='Field rb2A', value='V1', selected=False,x=144,y=72+0*36,buttonStyle='circle', borderStyle='solid',shape='circle', borderWidth=2, borderColor=red, fillColor=green, textColor=blue,forceBorder=True)
        af.radio(name='rb2A',tooltip='Field rb2A', value='V2', selected=True,x=144,y=72+1*36,buttonStyle='circle', borderStyle='solid',shape='circle', borderWidth=2, borderColor=red, fillColor=green, textColor=blue,forceBorder=True)
        af.radio(name='rb2B',tooltip='Field rb2B', value='V1', selected=False,x=144+36,y=72+0*36,buttonStyle='check', borderStyle='solid',shape='square', borderWidth=2, borderColor=green, fillColor=red, textColor=blue,forceBorder=True)
        af.radio(name='rb2B',tooltip='Field rb2B', value='V2', selected=False,x=144+36,y=72+1*36,buttonStyle='check', borderStyle='solid',shape='square', borderWidth=2, borderColor=green, fillColor=red, textColor=blue,forceBorder=True)
        af.radio(name='rb2C',tooltip='Field rb2C', value='V1', selected=False,x=144*2,y=72+0*36,buttonStyle='circle', borderStyle='bevelled',shape='circle', borderWidth=2, borderColor=green, fillColor=yellow, textColor=blue,forceBorder=True)
        af.radio(name='rb2C',tooltip='Field rb2C', value='V2', selected=True,x=144*2,y=72+1*36,buttonStyle='circle', borderStyle='inset',shape='circle', borderWidth=2, borderColor=magenta, fillColor=pink, textColor=blue,forceBorder=True)
        af.textfield(name='tf2A',tooltip='Field tf2A',value='Hello World',x=144*2+36,y=72+0*36, borderStyle='inset', borderWidth=2, borderColor=magenta, fillColor=pink, textColor=blue,forceBorder=True)
        af.textfield(name='tf2B',tooltip='Field tf2B',value='Hello World',x=144*2+36,y=72+2*36, borderStyle='inset', borderWidth=2, fontName='Courier-Bold', borderColor=magenta, fillColor=pink, textColor=blue,forceBorder=True)
        af.textfield(name='tf2C',tooltip='Field tf2C',value='Hello World',x=144*2+36,y=72+3*36, borderStyle='inset', borderWidth=0, fontName='Courier-Bold', borderColor=green, fillColor=red, textColor=black)
        canv.showPage()
        canv.translate(72,0)    #shift matrix
        #these are relative in changed matrix
        canv.drawString(4*72,800,'shifted relative')
        af.checkboxRelative(name='cb3A',tooltip='Field cb3A',checked=True,x=72,y=72,buttonStyle='circle',borderWidth=1, borderColor=red, fillColor=green, textColor=blue,forceBorder=True)
        af.checkboxRelative(name='cb3B',tooltip='Field cb3B',checked=True,x=72,y=72+36,buttonStyle='check',borderWidth=2, borderColor=green, fillColor=blue, textColor=red,forceBorder=True)
        af.checkboxRelative(name='cb3C',tooltip='Field cb3C',checked=True,x=72,y=72+2*36,buttonStyle='cross',borderWidth=2, borderColor=red, fillColor=green, textColor=blue,forceBorder=True)
        af.checkboxRelative(name='cb3D',tooltip='Field cb3D',checked=True,x=72,y=72+3*36,buttonStyle='star',borderWidth=2, borderColor=red, fillColor=green, textColor=blue,forceBorder=True)
        af.checkboxRelative(name='cb3E',tooltip='Field cb3E',checked=True,x=72,y=72+4*36,buttonStyle='diamond',borderStyle='bevelled', borderWidth=2, borderColor=red, fillColor=green, textColor=blue,forceBorder=True)
        af.checkboxRelative(name='cb3F',tooltip='Field cb3F',checked=True,x=72,y=72+5*36,buttonStyle='check', borderWidth=2, borderColor=red, fillColor=None, textColor=None,forceBorder=True)
        af.checkboxRelative(name='cb3H',tooltip='Field cb3H',checked=True,x=72,y=72+6*36,buttonStyle='check', borderStyle='underlined',borderWidth=2, borderColor=red, fillColor=None, textColor=None,forceBorder=True)
        af.checkboxRelative(name='cb3G',tooltip='Field cb3G',checked=True,x=72,y=72+7*36,buttonStyle='check', borderStyle='dashed',borderWidth=2, borderColor=red, fillColor=None, textColor=None,forceBorder=True)
        af.checkboxRelative(name='cb3I',tooltip='Field cb3I',checked=True,x=72,y=72+8*36,buttonStyle='check', borderStyle='inset',borderWidth=2, borderColor=red, fillColor=None, textColor=None,forceBorder=True)
        af.checkboxRelative(name='cb3J',tooltip='Field cb3J',checked=True,x=72,y=72+9*36,buttonStyle='check', borderStyle='solid',shape='circle', borderWidth=2, borderColor=red, fillColor=green, textColor=blue,forceBorder=True)
        af.checkboxRelative(name='cb3K',tooltip='Field cb3K',checked=True,x=72,y=72+10*36,buttonStyle='check', borderWidth=1, borderColor=None, fillColor=None, textColor=None,forceBorder=True)
        af.checkboxRelative(name='cb3L',tooltip='Field cb3L',checked=False,x=72,y=800,buttonStyle='check',borderWidth=None, borderColor=None, fillColor=None, textColor=None,forceBorder=True)
        af.checkboxRelative(name='cb3M',tooltip='Field cb3M',checked=False,x=72,y=600,buttonStyle='check',borderWidth=2, borderColor=blue, fillColor=None, textColor=None,forceBorder=True)
        af.radioRelative(name='rb3A',tooltip='Field rb3A', value='V1', selected=False,x=144,y=72+0*36,buttonStyle='circle', borderStyle='solid',shape='circle', borderWidth=2, borderColor=red, fillColor=green, textColor=blue,forceBorder=True)
        af.radioRelative(name='rb3A',tooltip='Field rb3A', value='V2', selected=True,x=144,y=72+1*36,buttonStyle='circle', borderStyle='solid',shape='circle', borderWidth=2, borderColor=red, fillColor=green, textColor=blue,forceBorder=True)
        af.radioRelative(name='rb3B',tooltip='Field rb3B', value='V1', selected=False,x=144+36,y=72+0*36,buttonStyle='check', borderStyle='solid',shape='square', borderWidth=2, borderColor=green, fillColor=red, textColor=blue,forceBorder=True)
        af.radioRelative(name='rb3B',tooltip='Field rb3B', value='V2', selected=False,x=144+36,y=72+1*36,buttonStyle='check', borderStyle='solid',shape='square', borderWidth=2, borderColor=green, fillColor=red, textColor=blue,forceBorder=True)
        af.radioRelative(name='rb3C',tooltip='Field rb3C', value='V1', selected=False,x=144*2,y=72+0*36,buttonStyle='circle', borderStyle='bevelled',shape='circle', borderWidth=2, borderColor=green, fillColor=yellow, textColor=blue,forceBorder=True)
        af.radioRelative(name='rb3C',tooltip='Field rb3C', value='V2', selected=True,x=144*2,y=72+1*36,buttonStyle='circle', borderStyle='inset',shape='circle', borderWidth=2, borderColor=magenta, fillColor=pink, textColor=blue,forceBorder=True)
        af.textfieldRelative(name='tf3A',tooltip='Field tf3A',value='Hello World',x=144*2+36,y=72+0*36, borderStyle='inset', borderWidth=2, borderColor=magenta, fillColor=pink, textColor=blue,forceBorder=True)
        af.textfieldRelative(name='tf3B',tooltip='Field tf3B',value='Hello World',x=144*2+36,y=72+2*36, borderStyle='inset', borderWidth=2, fontName='Courier-Bold', borderColor=magenta, fillColor=pink, textColor=blue,forceBorder=True)
        af.textfieldRelative(name='tf3C',tooltip='Field tf3C',value='Hello World',x=144*2+36,y=72+3*36, borderStyle='inset', borderWidth=0, fontName='Courier-Bold', borderColor=green, fillColor=red, textColor=black)
        canv.showPage()
        BS = ['solid','bevelled','inset','dashed','underlined']
        V = ['Av','B','Cv','D','Dv','E','F','G','Gv']
        ff = ['','edit']
        for i in range(500):
            x = 72+(i%3)*180
            y = 800 - int(i/3)*36
            if y<100: break
            value=V[i%len(V)]
            bW=i%3+1
            af.choice(name='CH%d'%i,value=value,x=x,y=y, width=72+bW, height=20+bW, borderStyle=BS[i%5], borderWidth=bW,
                fieldFlags=ff[i and i%9==0],
                borderColor=red, fillColor=green, textColor=blue,forceBorder=False,
                tooltip = 'CH%d value=%r' % (i,value),
                options=[('A','Av'),'B',('C','Cv'),('D','Dv'),'E',('F',),('G','Gv')])
        canv.showPage()
        BS = ['solid','bevelled','inset','dashed','underlined']
        V = ['Av','B','Cv','D','Dv','E','F','G','Gv','H']
        ff = ['','multiSelect']
        for i in range(500):
            x = 72+(i%3)*180
            y = 800 - int(i/3)*108 - 52
            if y<100: break
            v=i%len(V)
            value=[V[v]]
            if v>=5:
                v1 = V[3+i%5]
                if v1 not in value:
                    value.append(v1)
            bW=i%3+1
            fieldFlags=ff[len(value)>1]
            af.listbox(name='LB%d'%i,value=value,x=x,y=y, width=72+bW, height=72+bW, borderStyle=BS[i%5], borderWidth=bW,
                fieldFlags=fieldFlags,
                borderColor=red, fillColor=green, textColor=blue,forceBorder=False,
                tooltip = 'LB%d value=%r' % (i,value),
                options=[('A','Av'),'B',('C','Cv'),('D','Dv'),'E',('F',),('G','Gv'),'H'])
        canv.showPage()
        canv.save()

def makeSuite():
    return makeSuiteForClasses(PdfFormTestCase)

#noruntests
if __name__ == "__main__":
    unittest.TextTestRunner().run(makeSuite())
    printLocation()



#Autogenerated by ReportLab guiedit do not edit
from reportlab.graphics.shapes import _DrawingEditorMixin, Drawing, Group, Path
from reportlab.lib.colors import Color, CMYKColor, PCMYKColor

class ExplodedDrawing_AutoCloseDrawing(_DrawingEditorMixin,Drawing):
	def __init__(self,width=100.0,height=100.0,*args,**kw):
		Drawing.__init__(self,width,height,*args,**kw)
		self.transform = (1,0,0,1,0,0)
		v0=self._nn(Group())
		v0.transform = (1,0,0,-1,0,100)
		v1=v0._nn(Group())
		v1.transform = (1,0,0,-1,0,100)
		v1.add(Path(points=[10,10,10,90,20,90,20,10,30,10,30,90,40,90,40,10,50,10,50,90,60,90,60,10,70,10,70,90,80,90,80,10],operators=[0,1,1,1,3,0,1,1,1,0,1,1,1,3,0,1,1,1],isClipPath=0,autoclose=None,fillMode=1,fillColor=Color(0,0,1,1),fillOpacity=1,strokeColor=Color(1,0,0,1),strokeWidth=2,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))


if __name__=="__main__": #NORUNTESTS
	ExplodedDrawing_AutoCloseDrawing().save(formats=['pdf'],outDir='.',fnRoot=None)



#Autogenerated by ReportLab guiedit do not edit
from reportlab.graphics.shapes import _DrawingEditorMixin, Drawing, Group, Rect, String
from reportlab.lib.colors import Color, CMYKColor, PCMYKColor

class ExplodedDrawing_Drawing(_DrawingEditorMixin,Drawing):
	def __init__(self,width=756.284,height=240.20000000000002,*args,**kw):
		Drawing.__init__(self,width,height,*args,**kw)
		self.transform = (1,0,0,1,0,0)
		self.add(Rect(8,208.4,569.068,24.4,rx=0,ry=0,fillColor=Color(.827451,.827451,.827451,1),fillOpacity=None,strokeColor=Color(1,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(String(10,215.8,'Times-Roman: I should be totally horizontal and enclosed in a box and end in alphabetagamma ¢©®£ʥЖփאقকαβγ',textAnchor='start',fontName='Times-Roman',fontSize=12,fillColor=Color(0,0,0,1)))
		self.add(Rect(8,179,631.832,24.4,rx=0,ry=0,fillColor=Color(.827451,.827451,.827451,1),fillOpacity=None,strokeColor=Color(1,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(String(10,186.4,'Vera: I should be totally horizontal and enclosed in a box and end in alphabetagamma ¢©®£ʥЖփאقকαβγ',textAnchor='start',fontName='Vera',fontSize=12,fillColor=Color(0,0,0,1)))
		self.add(Rect(8,149.6,598.216,24.4,rx=0,ry=0,fillColor=Color(.827451,.827451,.827451,1),fillOpacity=None,strokeColor=Color(1,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(String(10,157,'Times-BoldItalic: I should be totally horizontal and enclosed in a box and end in alphabetagamma ¢©®£ʥЖփאقকαβγ',textAnchor='start',fontName='Times-BoldItalic',fontSize=12,fillColor=Color(0,0,0,1)))
		self.add(Rect(8,120.2,740.284,24.4,rx=0,ry=0,fillColor=Color(.827451,.827451,.827451,1),fillOpacity=None,strokeColor=Color(1,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(String(10,127.6,'Courier: I should be totally horizontal and enclosed in a box and end in alphabetagamma ¢©®£ʥЖփאقকαβγ',textAnchor='start',fontName='Courier',fontSize=12,fillColor=Color(0,0,0,1)))
		self.add(Rect(8,90.8,592.504,24.4,rx=0,ry=0,fillColor=Color(.827451,.827451,.827451,1),fillOpacity=None,strokeColor=Color(1,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(String(10,98.2,'Helvetica: I should be totally horizontal and enclosed in a box and end in alphabetagamma ¢©®£ʥЖփאقকαβγ',textAnchor='start',fontName='Helvetica',fontSize=12,fillColor=Color(0,0,0,1)))
		self.add(Rect(8,61.4,716.459,24.4,rx=0,ry=0,fillColor=Color(.827451,.827451,.827451,1),fillOpacity=None,strokeColor=Color(1,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(String(10,68.8,'VeraBd: I should be totally horizontal and enclosed in a box and end in alphabetagamma ¢©®£ʥЖփאقকαβγ',textAnchor='start',fontName='VeraBd',fontSize=12,fillColor=Color(0,0,0,1)))
		self.add(Rect(8,32,640.0762,24.4,rx=0,ry=0,fillColor=Color(.827451,.827451,.827451,1),fillOpacity=None,strokeColor=Color(1,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(String(10,39.4,'VeraIt: I should be totally horizontal and enclosed in a box and end in alphabetagamma ¢©®£ʥЖփאقকαβγ',textAnchor='start',fontName='VeraIt',fontSize=12,fillColor=Color(0,0,0,1)))
		self.add(Rect(8,2.6,712.334,24.4,rx=0,ry=0,fillColor=Color(.827451,.827451,.827451,1),fillOpacity=None,strokeColor=Color(1,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(String(10,10,'VeraBI: I should be totally horizontal and enclosed in a box and end in alphabetagamma ¢©®£ʥЖփאقকαβγ',textAnchor='start',fontName='VeraBI',fontSize=12,fillColor=Color(0,0,0,1)))


if __name__=="__main__": #NORUNTESTS
	ExplodedDrawing_Drawing().save(formats=['pdf'],outDir='.',fnRoot=None)



from reportlab.lib.testutils import setOutDir,makeSuiteForClasses, outputfile, printLocation, mockUrlRead
from unittest.mock import patch
setOutDir(__name__)
import unittest, sys, os
from reportlab.lib.testutils import testsFolder

def getFurl(fn):
    furl = fn.replace(os.sep,'/')
    if sys.platform=='win32' and furl[1]==':': furl = furl[0]+'|'+furl[2:]
    if furl[0]!='/': furl = '/'+furl
    return 'file://'+furl

def run():
    from reportlab.platypus import  BaseDocTemplate, PageTemplate, Image, Frame, PageTemplate, \
                                    ShowBoundaryValue, SimpleDocTemplate, FrameBG, Paragraph, \
                                    FrameBreak
    from reportlab.lib.colors import toColor
    from reportlab.lib.utils import haveImages, _RL_DIR, rl_isfile, open_for_read, fileName2FSEnc, asNative
    from reportlab.lib.styles import getSampleStyleSheet
    styleSheet = getSampleStyleSheet()
    if haveImages:
        _GIF = os.path.join(testsFolder,'pythonpowered.gif')
        if not rl_isfile(_GIF): _GIF = None
        _GAPNG = os.path.join(testsFolder,'gray-alpha.png')
        if not rl_isfile(_GAPNG): _GAPNG = None
    else:
        _GIF = None
    if _GIF: _GIFFSEnc=fileName2FSEnc(_GIF)
    if _GAPNG: _GAPNGFSEnc=fileName2FSEnc(_GAPNG)

    _JPG = os.path.join(testsFolder,'..','docs','images','lj8100.jpg')
    if not rl_isfile(_JPG): _JPG = None

    
    doc = SimpleDocTemplate(outputfile('test_platypus_images.pdf'))
    story=[FrameBG(color=toColor('lightblue'),start='frame-permanent'),
            ]
    if _GIF:
        story.append(Paragraph("Here is an Image flowable obtained from a string GIF filename.",styleSheet['Italic']))
        story.append(Image(_GIF))
        story.append(Paragraph( "Here is an Image flowable obtained from a utf8 GIF filename.", styleSheet['Italic']))
        #story.append(Image(fileName2FSEnc(_GIF)))
        story.append(Paragraph("Here is an Image flowable obtained from a string GIF file url.",styleSheet['Italic']))
        story.append(Image(getFurl(_GIF)))
        story.append(Paragraph("Here is an Image flowable obtained from an open GIF file.",styleSheet['Italic']))
        story.append(Image(open_for_read(_GIF,'b')))
        story.append(FrameBreak())
        img = Image('http://www.reportlab.com/rsrc/encryption.gif')
        story.append(Paragraph("Here is an Image flowable obtained from a string GIF http url.",styleSheet['Italic']))
        story.append(img)
        story.append(FrameBreak())

    if _GAPNG:
        story.append(Paragraph("Here is an Image flowable obtained from a string PNGA filename.",styleSheet['Italic']))
        story.append(Image('rltw-icon-tr.png'))
        story.append(Paragraph("Here is an Image flowable obtained from a string PNG filename.",styleSheet['Italic']))
        story.append(Image(_GAPNG))
        story.append(Paragraph( "Here is an Image flowable obtained from a utf8 PNG filename.", styleSheet['Italic']))
        #story.append(Image(fileName2FSEnc(_GAPNG)))
        story.append(Paragraph("Here is an Image flowable obtained from a string file PNG url.",styleSheet['Italic']))
        story.append(Image(getFurl(_GAPNG)))
        story.append(Paragraph("Here is an Image flowable obtained from an open PNG file.",styleSheet['Italic']))
        story.append(Image(open_for_read(_GAPNG,'b')))
        story.append(FrameBreak())

    if _JPG:
        img = Image(_JPG)
        story.append(Paragraph("Here is an JPEG Image flowable obtained from a JPEG filename.",styleSheet['Italic']))
        story.append(img)
        story.append(Paragraph("Here is an JPEG Image flowable obtained from an open JPEG file.",styleSheet['Italic']))
        img = Image(open_for_read(_JPG,'b'))
        story.append(img)
        story.append(FrameBreak())
    doc.build(story)

class PlatypusImagesTestCase(unittest.TestCase):
    @patch('reportlab.lib.utils.rlUrlRead', mockUrlRead)
    def test0(self):
        "Make a platypus document"
        run()

def makeSuite():
    return makeSuiteForClasses(PlatypusImagesTestCase)

#noruntests
if __name__ == "__main__":
    if '-debug' in sys.argv:
        run()
    else:
        unittest.TextTestRunner().run(makeSuite())
        printLocation()



#Autogenerated by ReportLab guiedit do not edit
from reportlab.graphics.shapes import _DrawingEditorMixin, Drawing, Group, String, Line
from reportlab.lib.colors import Color, CMYKColor, PCMYKColor

class ExplodedDrawing_Drawing(_DrawingEditorMixin,Drawing):
	def __init__(self,width=200,height=100,*args,**kw):
		Drawing.__init__(self,width,height,*args,**kw)
		self.transform = (1,0,0,1,0,0)
		self.add(String(55.22,85.585,'RED',textAnchor='end',fontName='Helvetica',fontSize=12,fillColor=Color(0,0,0,1)))
		self.add(String(20,85.585,'red',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(String(55.22,65.585,'GREEN',textAnchor='end',fontName='Helvetica',fontSize=12,fillColor=Color(0,0,0,1)))
		self.add(String(20,65.585,'green',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(String(55.22,45.585,'BLUE',textAnchor='end',fontName='Helvetica',fontSize=12,fillColor=Color(0,0,0,1)))
		self.add(String(20,45.585,'blue',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(String(130.22,85.585,'YELLOW',textAnchor='end',fontName='Helvetica',fontSize=12,fillColor=Color(0,0,0,1)))
		self.add(String(95,85.585,'yellow',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(String(130.22,65.585,'PINK',textAnchor='end',fontName='Helvetica',fontSize=12,fillColor=Color(0,0,0,1)))
		self.add(String(95,65.585,'pink',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(String(130.22,45.585,'BLACK',textAnchor='end',fontName='Helvetica',fontSize=12,fillColor=Color(0,0,0,1)))
		self.add(String(95,45.585,'black',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(String(205.22,85.585,'WHITE',textAnchor='end',fontName='Helvetica',fontSize=12,fillColor=Color(0,0,0,1)))
		self.add(String(170,85.585,'white',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(Line(57.22,88,67.22,88,strokeColor=Color(1,0,0,1),strokeWidth=2,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(57.22,68,67.22,68,strokeColor=Color(0,.501961,0,1),strokeWidth=2,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(57.22,48,67.22,48,strokeColor=Color(0,0,1,1),strokeWidth=2,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(132.22,88,142.22,88,strokeColor=Color(1,1,0,1),strokeWidth=2,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(132.22,68,142.22,68,strokeColor=Color(1,.752941,.796078,1),strokeWidth=2,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(132.22,48,142.22,48,strokeColor=Color(0,0,0,1),strokeWidth=2,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(207.22,88,217.22,88,strokeColor=Color(1,1,1,1),strokeWidth=2,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))


if __name__=="__main__": #NORUNTESTS
	ExplodedDrawing_Drawing().save(formats=['pdf'],outDir='.',fnRoot=None)



#Copyright ReportLab Europe Ltd. 2000-2017
#see license.txt for license details
#history https://hg.reportlab.com/hg-public/reportlab/log/tip/tools/docco/yaml2pdf.py
# yaml2pdf - turns stuff in Yet Another Markup Language
# into PDF documents.  Very crude - it assumes a
# doc template and stylesheet (hard coded for now)
# and basically cranks out paragraphs in each style
"""yaml2pdf.py - converts Yet Another Markup Language
to reasonable PDF documents.  This is ReportLab's
basic documentation tool.

Usage:
.  "yaml2pdf.py filename.ext" will create "filename.pdf"
"""

import sys
import os
import imp

from . import yaml
from .rltemplate import RLDocTemplate
from reportlab.lib.styles import ParagraphStyle
from reportlab.lib.enums import *
from reportlab.lib.pagesizes import A4
from reportlab.platypus import *
from reportlab.lib import colors
from reportlab.lib.units import inch


from .stylesheet import getStyleSheet


def run(infilename, outfilename):
    p = yaml.Parser()
    results = p.parseFile(infilename)

    ss = getStyleSheet()

    #now make flowables from the results
    story = []
    for thingy in results:
        typ = thingy[0]
        if typ == 'Paragraph':
            (typ2, stylename, text) = thingy
            if stylename == 'bu':
                bulletText='\267'
            else:
                bulletText=None
            try:
                style = ss[stylename]
            except KeyError:
                print('Paragraph style "%s" not found in stylesheet, using Normal instead' % stylename)
                style = ss['Normal']
            story.append(Paragraph(text, style, bulletText=bulletText))
        elif typ == 'Preformatted':
            (typ2, stylename, text) = thingy
            try:
                style = ss[stylename]
            except KeyError:
                print('Preformatted style "%s" not found in stylesheet, using Normal instead' % stylename)
                style = ss['Normal']
            story.append(Preformatted(text, style, bulletText=bulletText))
        elif typ == 'Image':
            filename = thingy[1]
            img = Image(filename)
            story.append(img)
        elif typ == 'PageBreak':
            story.append(PageBreak())
        elif typ == 'VSpace':
            height = thingy[1]
            story.append(Spacer(0, height))
        elif typ == 'NextPageTemplate':
            story.append(NextPageTemplate(thingy[1]))
        elif typ == 'Custom':
            # go find it
            searchPath = [os.getcwd()+'\\']
            (typ2, moduleName, funcName) = thingy
            found = imp.find_module(moduleName, searchPath)
            assert found, "Custom object module %s not found" % moduleName
            (file, pathname, description) = found
            mod = imp.load_module(moduleName, file, pathname, description)

            #now get the function
            func = getattr(mod, funcName)
            story.append(func())

        else:
            print('skipping',typ, 'for now')


    #print it
    doc = RLDocTemplate(outfilename, pagesize=A4)
    doc.build(story)

if __name__ == '__main__': #NORUNTESTS
    if len(sys.argv) == 2:
        infilename = sys.argv[1]
        outfilename = os.path.splitext(infilename)[0] + '.pdf'
        if os.path.isfile(infilename):
            run(infilename, outfilename)
        else:
            print('File not found %s' % infilename)
    else:
        print(__doc__)



#How to add bleed to a page in this case 6mm to a landscape A4
from reportlab.lib import units, pagesizes
from reportlab.pdfgen.canvas import Canvas
import sys, os, glob, time
bleedX = 6*units.mm
bleedY = 6*units.mm
pageWidth, pageHeight = pagesizes.landscape(pagesizes.A4)
def process_pdf(c,infn,prefix='PageForms'):
    from rlextra.pageCatcher import pageCatcher
    names, data = pageCatcher.storeFormsInMemory(open(infn,'rb').read(),prefix=prefix,all=1)
    names = pageCatcher.restoreFormsInMemory(data,c)
    del data
    for i in range(len(names)):
        thisname = names[i]
        c.saveState()
        c.translate(bleedX,bleedY)
        c.doForm(thisname)
        c.restoreState()
        c.showPage()

def main():
    for infn in sys.argv[1:]:
        outfn = 'bleeding_'+os.path.basename(infn)
        c = Canvas(outfn,pagesize=(pageWidth+2*bleedX,pageHeight+2*bleedY))
        process_pdf(c,infn)
        c.save()
if __name__=='__main__':
    main()



#!/bin/env python
import os, sys, traceback
def _genAll(verbose=1):
    from reportlab.lib.testutils import setOutDir
    setOutDir(__name__)
    from reportlab.lib.testutils import testsFolder
    topDir=os.path.dirname(testsFolder)
    L = [os.path.join(topDir,f) for f in (
            #'docs/reference/genreference.py',
            'docs/userguide/genuserguide.py',
            #'tools/docco/graphdocpy.py',
            )   
        ]
    for f in ('src/rl_addons/pyRXP/docs/PyRXP_Documentation.rml',
            ):
        f = os.path.join(topDir,f)
        if os.path.isfile(f):
            L += [f]
            break
    for p in L:
        os.chdir(os.path.dirname(p))
        if p[-4:]=='.rml':
            try:
                from rlextra.rml2pdf.rml2pdf import main
                main(exe=0,fn=[os.path.basename(p)], quiet=not verbose, outDir=d)
            except:
                if verbose: traceback.print_exc()
        else:
            cmd = '"%s" %s %s' % (sys.executable,os.path.basename(p), not verbose and '-s' or '')
            if verbose: print(cmd)
            os.system(cmd)

"""Runs the manual-building scripts"""
if __name__=='__main__':
    #need a quiet mode for the test suite
    if '-s' in sys.argv:  # 'silent
        verbose = 0
    else:
        verbose = 1
    d = os.path.dirname(sys.argv[0])
    if not d:
        d = os.getcwd()
    elif not os.path.isabs(d):
        d = os.path.abspath(d)
    sys.path.insert(0,os.path.dirname(d))
    _genAll(verbose)



"""This executes tests defined outside the normal test suite.
See docstring for class ExternalTestCase for more information.
"""
from reportlab.lib.testutils import setOutDir,SecureTestCase, printLocation
setOutDir(__name__)
import os, string, fnmatch, re, sys, unittest
EXTRA_FILE = 'extra.txt'

class ExternalTestCase(SecureTestCase):
    """Test case starting cases external to the normal RL suite.

    This test case runs additional test cases defined in external
    modules which must also be using the unittest framework.
    These additional modules must be defined in a file named
    'extra.txt' which needs to be located in the tests
    folder and contains one path per line.

    The paths of these modules can contain stuff from the Unix
    world like '~', '.', '..' and '$HOME' and can have absolute
    or relative paths. If they are relative they start from the
    tests folder.

    This is a sample 'extra.txt' file:

        foo.py
        ./foo.py
        bar/foo.py
        ../foo.py
        /bar/foo.py
        ~/foo.py
        ~/bar/foo.py
        ~/../foo.py
        $HOME/bar/foo.py
    """

    def test0(self):
        "Execute external test cases."

        cwd = os.getcwd()

        # look for a file named 'extra.txt' in test directory,
        # exit if not found
        from reportlab.lib.testutils import RL_HOME
        extraFilename = os.path.join(RL_HOME, 'test', EXTRA_FILE)
        if not os.path.exists(extraFilename):
            return

        # read module paths from file
        extraModulenames = open(extraFilename).readlines()
        extraModulenames = list(map(string.strip, extraModulenames))

        # expand pathnames as much as possible
        for f in extraModulenames:
            if f == '':
                continue
            if f[0] == '#':
                continue
            f = os.path.expanduser(f)
            f = os.path.expandvars(f)
            f = os.path.abspath(f)

            if os.path.exists(f):
                # look for a makeSuite function and execute it if present
                folder = os.path.abspath(os.path.dirname(f))
                modname = os.path.splitext(os.path.basename(f))[0]
                os.chdir(folder)
                sys.path.insert(0, folder)

                module = __import__(modname) # seems to fail sometimes...
                if 'makeSuite' in dir(module):
                    print("running", f)
                    testSuite = module.makeSuite()
                    unittest.TextTestRunner().run(testSuite)
                os.chdir(cwd)
                sys.path = sys.path[1:]


def makeSuite():
    suite = unittest.TestSuite()
    loader = unittest.TestLoader()
    if sys.platform[:4] != 'java':
        suite.addTest(loader.loadTestsFromTestCase(ExternalTestCase))

    return suite


#noruntests
if __name__ == "__main__":
    if len(sys.argv) > 1:
        EXTRA_FILE = sys.argv[1]
        assert os.path.isfile(EXTRA_FILE), 'file %s not found!' % EXTRA_FILE
    # otherwise, extra.txt will be used
    unittest.TextTestRunner().run(makeSuite())
    printLocation()



#Copyright ReportLab Europe Ltd. 2000-2017
#see license.txt for license details
"""Tests pleaseTurnOver, pageBreakBefore, frameBreakBefore, keepWithNext...
"""
__version__='3.3.0'
from reportlab.lib.testutils import setOutDir,makeSuiteForClasses, outputfile, printLocation
setOutDir(__name__)
import sys, unittest
from reportlab.platypus.flowables import Flowable, PTOContainer, KeepInFrame
from reportlab.platypus import FrameBG
from reportlab.lib.units import cm
from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle
from reportlab.lib.colors import toColor, black
from reportlab.lib.enums import TA_LEFT, TA_RIGHT, TA_CENTER, TA_JUSTIFY
from reportlab.platypus.paragraph import Paragraph
from reportlab.platypus.tables import Table
from reportlab.platypus.frames import Frame
from reportlab.platypus.doctemplate import PageTemplate, BaseDocTemplate, FrameBreak

def myMainPageFrame(canvas, doc):
    "The page frame used for all PDF documents."

    canvas.saveState()
    canvas.setFont('Times-Roman', 12)
    pageNumber = canvas.getPageNumber()
    canvas.drawString(10*cm, cm, str(pageNumber))
    canvas.restoreState()

def _showDoc(fn,story):
    pageTemplate = PageTemplate('normal', [Frame(72, 440, 170, 284, id='F1'),
                            Frame(326, 440, 170, 284, id='F2'),
                            Frame(72, 72, 170, 284, id='F3'),
                            Frame(326, 72, 170, 284, id='F4'),
                            ], myMainPageFrame)
    doc = BaseDocTemplate(outputfile(fn),
            pageTemplates = pageTemplate,
            showBoundary = 1,
            )
    doc.multiBuild(story)

text2 ='''We have already seen that the natural general principle that will
subsume this case cannot be arbitrary in the requirement that branching
is not tolerated within the dominance scope of a complex symbol.
Notice, incidentally, that the speaker-hearer's linguistic intuition is
to be regarded as the strong generative capacity of the theory.  A
consequence of the approach just outlined is that the descriptive power
of the base component does not affect the structure of the levels of
acceptability from fairly high (e.g. (99a)) to virtual gibberish (e.g.
(98d)).  By combining adjunctions and certain deformations, a
descriptively adequate grammar cannot be arbitrary in the strong
generative capacity of the theory.'''

text1='''
On our assumptions, a descriptively adequate grammar delimits the strong
generative capacity of the theory.  For one thing, the fundamental error
of regarding functional notions as categorial is to be regarded as a
corpus of utterance tokens upon which conformity has been defined by the
paired utterance test.  A majority  of informed linguistic specialists
agree that the appearance of parasitic gaps in domains relatively
inaccessible to ordinary extraction is necessary to impose an
interpretation on the requirement that branching is not tolerated within
the dominance scope of a complex symbol.  It may be, then, that the
speaker-hearer's linguistic intuition appears to correlate rather
closely with the ultimate standard that determines the accuracy of any
proposed grammar.  Analogously, the notion of level of grammaticalness
may remedy and, at the same time, eliminate a general convention
regarding the forms of the grammar.'''
    
text0 = '''To characterize a linguistic level L,
this selectionally introduced contextual
feature delimits the requirement that
branching is not tolerated within the
dominance scope of a complex
symbol. Notice, incidentally, that the
notion of level of grammaticalness
does not affect the structure of the
levels of acceptability from fairly high
(e.g. (99a)) to virtual gibberish (e.g.
(98d)). Suppose, for instance, that a
subset of English sentences interesting
on quite independent grounds appears
to correlate rather closely with an
important distinction in language use.
Presumably, this analysis of a
formative as a pair of sets of features is
not quite equivalent to the system of
base rules exclusive of the lexicon. We
have already seen that the appearance
of parasitic gaps in domains relatively
inaccessible to ordinary extraction
does not readily tolerate the strong
generative capacity of the theory.'''

def _ptoTestCase(self):
    """PTO stands for Please Turn Over and is a means for
    specifying content to be inserted when stuff goes over a page.
    This makes one long multi-page paragraph."""

    # Build story.
    story = []
    def fbreak(story=story):
        story.append(FrameBreak())

    styleSheet = getSampleStyleSheet()
    H1 = styleSheet['Heading1']
    H1.pageBreakBefore = 0
    H1.keepWithNext = 0

    bt = styleSheet['BodyText']
    pto = ParagraphStyle('pto',parent=bt)
    pto.alignment = TA_RIGHT
    pto.fontSize -= 1
    def ColorParagraph(c,text,style):
        return Paragraph('<para color="%s">%s</para>' % (c,text),style)

    def ptoblob(blurb,content,trailer=None,header=None, story=story, H1=H1):
        if type(content) not in (type([]),type(())): content = [content]
        story.append(PTOContainer([Paragraph(blurb,H1)]+list(content),trailer,header))

    t0 = [ColorParagraph('blue','Please turn over', pto )]
    h0 = [ColorParagraph('blue','continued from previous page', pto )]
    t1 = [ColorParagraph('red','Please turn over(inner)', pto )]
    h1 = [ColorParagraph('red','continued from previous page(inner)', pto )]
    ptoblob('First Try at a PTO',[Paragraph(text0,bt)],t0,h0)
    fbreak()
    c1 = Table([('alignment', 'align\012alignment'),
                ('bulletColor', 'bulletcolor\012bcolor'),
                ('bulletFontName', 'bfont\012bulletfontname'),
                ('bulletFontSize', 'bfontsize\012bulletfontsize'),
                ('bulletIndent', 'bindent\012bulletindent'),
                ('firstLineIndent', 'findent\012firstlineindent'),
                ('fontName', 'face\012fontname\012font'),
                ('fontSize', 'size\012fontsize'),
                ('leading', 'leading'),
                ('leftIndent', 'leftindent\012lindent'),
                ('rightIndent', 'rightindent\012rindent'),
                ('spaceAfter', 'spaceafter\012spacea'),
                ('spaceBefore', 'spacebefore\012spaceb'),
                ('textColor', 'fg\012textcolor\012color')],
            style = [
                ('VALIGN',(0,0),(-1,-1),'TOP'),
                ('INNERGRID', (0,0), (-1,-1), 0.25, black),
                ('BOX', (0,0), (-1,-1), 0.25, black),
                ],
            )
    ptoblob('PTO with a table inside',c1,t0,h0)
    fbreak()
    ptoblob('A long PTO',[Paragraph(text0+' '+text1,bt)],t0,h0)
    fbreak()
    ptoblob('2 PTO (inner split)',[ColorParagraph('pink',text0,bt),PTOContainer([ColorParagraph(black,'Inner Starts',H1),ColorParagraph('yellow',text2,bt),ColorParagraph('black','Inner Ends',H1)],t1,h1),ColorParagraph('magenta',text1,bt)],t0,h0)
    _showDoc('test_platypus_pto.pdf',story)

def _KeepInFrameTestCase(self,mode,offset=0):
    story = []
    def fbreak(story=story):
        story.append(FrameBreak())
    styleSheet = getSampleStyleSheet()
    H1 = styleSheet['Heading1']
    H1.pageBreakBefore = 0
    H1.keepWithNext = 0
    bt = styleSheet['BodyText']
    def subStory(texts,fbg=0):
        style = [
            ('VALIGN',(0,0),(-1,-1),'TOP'),
            ('INNERGRID', (0,0), (-1,-1), 0.25, black),
            ('BOX', (0,0), (-1,-1), 0.25, black),
            ]
        if fbg:
            fbg1 = [FrameBG(start=0,color=toColor('limegreen'))]
            fbg0 = [FrameBG(start=0)]
        else:
            fbg0 = fbg1 = []
        return ([Paragraph(t,bt) for t in texts]
                +fbg1+[Table([('alignment', a.lower())],style = style,hAlign=a)
                    for a in ('LEFT','RIGHT','CENTER')]+fbg0)
    def allModesKIF(just,ifb=True,width=170):
        if ifb: fbreak()
        story.append(KeepInFrame(width-offset,284-offset,subStory(texts=(text0,)),mode=mode,hAlign=just))
        fbreak()
        story.append(KeepInFrame(width-offset,284-offset,subStory(texts=(text0,text1)),mode=mode,hAlign=just))
        fbreak()
        story.append(KeepInFrame(width-offset,284-offset,subStory(texts=(text0,text1,text2)),mode=mode,hAlign=just))
        fbreak()
        story.append(KeepInFrame(width-offset,284-offset,subStory(texts=(text0,text1,text2),fbg=1),mode=mode,hAlign=just))
    allModesKIF('LEFT',False)
    allModesKIF('LEFT',width=100)
    allModesKIF('CENTRE',width=100)
    allModesKIF('RIGHT',width=100)
    _showDoc('test_platypus_KeepInFrame%s.pdf'%mode,story)

class TestCases(unittest.TestCase):
    "Test multi-page splitting of paragraphs (eyeball-test)."
    def test0(self):
        _ptoTestCase(self)
    def test1(self):
        _KeepInFrameTestCase(self,mode="shrink")
    def test2(self):
        _KeepInFrameTestCase(self,mode="overflow")
    def test3(self):
        _KeepInFrameTestCase(self,mode="truncate")
    def test4(self):
        from reportlab.platypus.doctemplate import LayoutError
        self.assertRaises(LayoutError, _KeepInFrameTestCase,*(self,"error"))
    def test5(self):
        _KeepInFrameTestCase(self,"shrink",0)

def makeSuite():
    return makeSuiteForClasses(TestCases)

#noruntests
if __name__ == "__main__": #NORUNTESTS
    if 'debug' in sys.argv:
        _KeepInFrameTestCase(None)
    else:
        unittest.TextTestRunner().run(makeSuite())
        printLocation()



#Autogenerated by ReportLab guiedit do not edit
from reportlab.graphics.shapes import _DrawingEditorMixin, Drawing, Group, Rect, Line, String
from reportlab.lib.colors import Color, CMYKColor, PCMYKColor

class ExplodedDrawing_Drawing(_DrawingEditorMixin,Drawing):
	def __init__(self,width=400,height=200,*args,**kw):
		Drawing.__init__(self,width,height,*args,**kw)
		self.transform = (1,0,0,1,0,0)
		self.add(Rect(50,50,300,125,rx=0,ry=0,fillColor=None,fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(50,53.47222,50,13.88889,rx=0,ry=0,fillColor=Color(1,0,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(50,115.9722,300,13.88889,rx=0,ry=0,fillColor=Color(1,0,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(50,67.36111,100,13.88889,rx=0,ry=0,fillColor=Color(0,.501961,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(50,129.8611,250,13.88889,rx=0,ry=0,fillColor=Color(0,.501961,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(50,81.25,150,13.88889,rx=0,ry=0,fillColor=Color(0,0,1,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(50,143.75,200,13.88889,rx=0,ry=0,fillColor=Color(0,0,1,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(50,95.13889,200,13.88889,rx=0,ry=0,fillColor=Color(1,0,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(50,157.6389,150,13.88889,rx=0,ry=0,fillColor=Color(1,0,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(50,50,50,175,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(50,50,45,50,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(50,112.5,45,112.5,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(50,175,45,175,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,45,81.25)
		v0.add(String(-20,-4,'Ying',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,45,143.75)
		v0.add(String(-21.66,-4,'Yang',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(Line(50,50,350,50,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(50,50,50,45,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(125,50,125,45,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(200,50,200,45,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(275,50,275,45,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(350,50,350,45,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=10,strokeDashArray=None,strokeOpacity=None))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,50,45)
		v0.add(String(-2.5,-10,'0',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,125,45)
		v0.add(String(-5,-10,'15',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,200,45)
		v0.add(String(-5,-10,'30',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,275,45)
		v0.add(String(-5,-10,'45',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,350,45)
		v0.add(String(-5,-10,'60',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))


if __name__=="__main__": #NORUNTESTS
	ExplodedDrawing_Drawing().save(formats=['pdf'],outDir='.',fnRoot=None)



#Autogenerated by ReportLab guiedit do not edit
from reportlab.graphics.shapes import _DrawingEditorMixin, Drawing, Group, Rect, String, Line
from reportlab.lib.colors import Color, CMYKColor, PCMYKColor

class ExplodedDrawing_Drawing(_DrawingEditorMixin,Drawing):
	def __init__(self,width=400,height=200,*args,**kw):
		Drawing.__init__(self,width,height,*args,**kw)
		self.transform = (1,0,0,1,0,0)
		self.add(Rect(225,22.5,-93.75,10,rx=0,ry=0,fillColor=Color(1,0,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(225,37.5,18.75,10,rx=0,ry=0,fillColor=Color(1,0,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(225,52.5,31.25,10,rx=0,ry=0,fillColor=Color(1,0,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(225,67.5,62.5,10,rx=0,ry=0,fillColor=Color(1,0,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(225,82.5,50,10,rx=0,ry=0,fillColor=Color(1,0,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(225,97.5,43.75,10,rx=0,ry=0,fillColor=Color(1,0,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(225,112.5,25,10,rx=0,ry=0,fillColor=Color(1,0,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(225,127.5,6.25,10,rx=0,ry=0,fillColor=Color(1,0,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(225,142.5,62.5,10,rx=0,ry=0,fillColor=Color(1,0,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Rect(225,157.5,18.75,10,rx=0,ry=0,fillColor=Color(1,0,0,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,121.25,27.5)
		v0.add(String(-6.837,-2.4,'-1.50',textAnchor='middle',fontName='Helvetica',fontSize=6,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,253.75,42.5)
		v0.add(String(5.838,-2.4,'0.30',textAnchor='middle',fontName='Helvetica',fontSize=6,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,266.25,57.5)
		v0.add(String(5.838,-2.4,'0.50',textAnchor='middle',fontName='Helvetica',fontSize=6,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,297.5,72.5)
		v0.add(String(5.838,-2.4,'1.00',textAnchor='middle',fontName='Helvetica',fontSize=6,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,285,87.5)
		v0.add(String(5.838,-2.4,'0.80',textAnchor='middle',fontName='Helvetica',fontSize=6,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,278.75,102.5)
		v0.add(String(5.838,-2.4,'0.70',textAnchor='middle',fontName='Helvetica',fontSize=6,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,260,117.5)
		v0.add(String(5.838,-2.4,'0.40',textAnchor='middle',fontName='Helvetica',fontSize=6,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,241.25,132.5)
		v0.add(String(5.838,-2.4,'0.10',textAnchor='middle',fontName='Helvetica',fontSize=6,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,297.5,147.5)
		v0.add(String(5.838,-2.4,'1.00',textAnchor='middle',fontName='Helvetica',fontSize=6,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,253.75,162.5)
		v0.add(String(5.838,-2.4,'0.30',textAnchor='middle',fontName='Helvetica',fontSize=6,fillColor=Color(0,0,0,1)))
		self.add(Line(225,20,225,170,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,55,27.5)
		v0.add(String(0,-2.4,'UK Equities',textAnchor='start',fontName='Helvetica',fontSize=6,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,55,42.5)
		v0.add(String(0,-2.4,'US Equities',textAnchor='start',fontName='Helvetica',fontSize=6,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,55,57.5)
		v0.add(String(0,-2.4,'European Equities',textAnchor='start',fontName='Helvetica',fontSize=6,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,55,72.5)
		v0.add(String(0,-2.4,'Japanese Equities',textAnchor='start',fontName='Helvetica',fontSize=6,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,55,87.5)
		v0.add(String(0,-2.4,'Pacific (ex Japan) Equities',textAnchor='start',fontName='Helvetica',fontSize=6,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,55,102.5)
		v0.add(String(0,-2.4,'Emerging Markets Equities',textAnchor='start',fontName='Helvetica',fontSize=6,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,55,117.5)
		v0.add(String(0,-2.4,'UK Bonds',textAnchor='start',fontName='Helvetica',fontSize=6,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,55,132.5)
		v0.add(String(0,-2.4,'Overseas Bonds',textAnchor='start',fontName='Helvetica',fontSize=6,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,55,147.5)
		v0.add(String(0,-2.4,'UK Index-Linked',textAnchor='start',fontName='Helvetica',fontSize=6,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,55,162.5)
		v0.add(String(0,-2.4,'Cash',textAnchor='start',fontName='Helvetica',fontSize=6,fillColor=Color(0,0,0,1)))


if __name__=="__main__": #NORUNTESTS
	ExplodedDrawing_Drawing().save(formats=['pdf'],outDir='.',fnRoot=None)



#Autogenerated by ReportLab guiedit do not edit
from reportlab.graphics.shapes import _DrawingEditorMixin, Drawing, Group, Wedge, String, Line
from reportlab.lib.colors import Color, CMYKColor, PCMYKColor

class ExplodedDrawing_Drawing(_DrawingEditorMixin,Drawing):
	def __init__(self,width=400,height=200,*args,**kw):
		Drawing.__init__(self,width,height,*args,**kw)
		self.transform = (1,0,0,1,0,0)
		self.add(Wedge(200,100,75,-278.8462,-85,yradius=75,annular=False,fillColor=Color(.27451,.509804,.705882,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=1,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Wedge(200,100,75,-306.5385,-278.8462,yradius=75,annular=False,fillColor=Color(.847059,.74902,.847059,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=1,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Wedge(200,100,75,-334.2308,-306.5385,yradius=75,annular=False,fillColor=Color(.392157,.584314,.929412,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=1,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Wedge(200,100,75,-361.9231,-334.2308,yradius=75,annular=False,fillColor=Color(.690196,.768627,.870588,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=1,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Wedge(200,100,75,-389.6154,-361.9231,yradius=75,annular=False,fillColor=Color(.498039,1,.831373,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=1,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Wedge(200,100,75,-445,-389.6154,yradius=75,annular=False,fillColor=Color(.372549,.619608,.627451,1),fillOpacity=None,strokeColor=Color(0,0,0,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=1,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(String(110,103.0202,'example1',textAnchor='end',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(String(290,183.0331,'example2',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(String(290,157.3868,'example3',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(String(290,118.5938,'example4',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(String(290,75.54129,'example5',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(String(290,24.2575,'example6',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		self.add(Line(125.0422,102.5168,117.5211,106.0202,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(117.5211,106.0202,110,106.0202,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(228.9337,169.1942,259.4668,186.0331,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(259.4668,186.0331,290,186.0331,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(257.7757,147.8223,273.8878,160.3868,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(273.8878,160.3868,290,160.3868,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(273.3819,115.4949,281.691,121.5938,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(281.691,121.5938,290,121.5938,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(272.1773,79.61774,281.0887,78.54129,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(281.0887,78.54129,290,78.54129,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(240.5096,36.88125,265.2548,27.2575,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(265.2548,27.2575,290,27.2575,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))


if __name__=="__main__": #NORUNTESTS
	ExplodedDrawing_Drawing().save(formats=['pdf'],outDir='.',fnRoot=None)



#Copyright ReportLab Europe Ltd. 2000-2017
#see license.txt for license details
__version__ = '$Id$'
from tools.docco.rl_doc_utils import *
from reportlab.platypus.tableofcontents import TableOfContents
import reportlab
from reportlab.lib.utils import TimeStamp

title("ReportLab PDF Library")
title("User Guide")
centred('ReportLab Version ' + reportlab.Version)
centred(TimeStamp().datetime.strftime('Document generated on %Y/%m/%d %H:%M:%S %Z'))

nextTemplate("TOC")

headingTOC()

toc = TableOfContents()
PS = ParagraphStyle
toc.levelStyles = [
    PS(fontName='Times-Bold', fontSize=14, name='TOCHeading1', leftIndent=20, firstLineIndent=-20, spaceBefore=5, leading=16),
    PS(fontSize=12, name='TOCHeading2', leftIndent=40, firstLineIndent=-20, spaceBefore=0, leading=12),
    PS(fontSize=10, name='TOCHeading3', leftIndent=60, firstLineIndent=-20, spaceBefore=0, leading=12),
    PS(fontSize=10, name='TOCHeading4', leftIndent=100, firstLineIndent=-20, spaceBefore=0, leading=12),
]
getStory().append(toc)

nextTemplate("Normal")

########################################################################
#
#               Chapter 1
#
########################################################################


heading1("Introduction")


heading2("About this document")
disc("""This document is an introduction to the ReportLab PDF library.
Some previous programming experience
is presumed and familiarity with the Python Programming language is
recommended.  If you are new to Python, we tell you in the next section
where to go for orientation.
""")

disc("""
This manual does not cover 100% of the features, but should explain all
the main concepts and help you get started, and point you at other
learning resources.
After working your way through this, you should be ready to begin
writing programs to produce sophisticated reports.
""")

disc("""In this chapter, we will cover the groundwork:""")
bullet("What is ReportLab all about, and why should I use it?")
bullet("What is Python?")
bullet("How do I get everything set up and running?")

todo("""
We need your help to make sure this manual is complete and helpful.
Please send any feedback to our user mailing list,
which is signposted from <a href="http://www.reportlab.com/">www.reportlab.com</a>.
""")

heading2("What is the ReportLab PDF Library?")
disc("""This is a software library that lets you directly
create documents in Adobe's Portable Document Format (PDF) using
the Python programming language.   It also creates charts and data graphics
in various bitmap and vector formats as well as PDF.""")

disc("""PDF is the global standard for electronic documents. It
supports high-quality printing yet is totally portable across
platforms, thanks to the freely available Acrobat Reader.  Any
application which previously generated hard copy reports or driving a printer
can benefit from making PDF documents instead; these can be archived,
emailed, placed on the web, or printed out the old-fashioned way.
However, the PDF file format is a complex
indexed binary format which is impossible to type directly.
The PDF format specification is more than 600 pages long and
PDF files must provide precise byte offsets -- a single extra
character placed anywhere in a valid PDF document can render it
invalid.  This makes it harder to generate than HTML.""")

disc("""Most of the world's PDF documents have been produced
by Adobe's Acrobat tools, or rivals such as JAWS PDF Creator, which act
as 'print drivers'.  Anyone wanting to automate PDF production would
typically use a product like Quark, Word or Framemaker running in a loop
with macros or plugins, connected to Acrobat. Pipelines of several
languages and products can be slow and somewhat unwieldy.
""")


disc("""The ReportLab library directly creates PDF based on
your graphics commands.  There are no intervening steps.  Your applications
can generate reports extremely fast - sometimes orders
of magnitude faster than traditional report-writing
tools.   This approach is shared by several other libraries - PDFlib for C,
iText for Java, iTextSharp for .NET and others.  However, The ReportLab library
differs in that it can work at much higher levels, with a full featured engine
for laying out documents complete with tables and charts.  """)


disc("""In addition, because you are writing a program
in a powerful general purpose language, there are no
restrictions at all on where you get your data from,
how you transform it, and the kind of output
you can create.  And you can reuse code across
whole families of reports.""")

disc("""The ReportLab library is expected to be useful
in at least the following contexts:""")
bullet("Dynamic PDF generation on the web")
bullet("High-volume corporate reporting and database publishing")
bullet("""An embeddable print engine for other applications, including
a 'report language' so that users can customize their own reports. <i>
This is particularly relevant to cross-platform apps which cannot
rely on a consistent printing or previewing API on each operating
system</i>.""")
bullet("""A 'build system' for complex documents with charts, tables
and text such as management accounts, statistical reports and
scientific papers """)
bullet("""Going from XML to PDF in one step""")


heading2("ReportLab's commercial software")
disc("""
The ReportLab library forms the foundation of our commercial solution for
PDF generation, Report Markup Language (RML).  This is available for evaluation
on our web site with full documentation.   We believe that RML is the fastest
and easiest way to develop rich PDF workflows.  You work in a markup language
at a similar level to HTML, using your favorite templating system to populate
an RML document; then call our rml2pdf API function to generate a PDF.  It's
what ReportLab staff use to build all of the solutions you can see on reportlab.com.
Key differences:
""")
bullet("""Fully documented with two manuals, a formal specification (the DTD) and extensive self-documenting tests.  (By contrast, we try to make sure the open source documentation isn't wrong, but we don't always keep up with the code)""")
bullet("""Work in high-level markup rather than constructing graphs of Python objects """)
bullet("""Requires no Python expertise - your colleagues may thank you after you've left!'""")
bullet("""Support for vector graphics and inclusion of other PDF documents""")
bullet("""Many more useful features expressed with a single tag, which would need a lot
of coding in the open source package""")
bullet("""Commercial support is included""")


disc("""
We ask open source developers to consider trying out RML where it is appropriate.
You can register on our site and try out a copy before buying.
The costs are reasonable and linked to the volume of the project, and the revenue
helps us spend more time developing this software.""")


heading2("What is Python?")
disc("""
Python is an <i>interpreted, interactive, object-oriented</i> programming language. It is often compared to Tcl, Perl,
Scheme or Java.
""")

disc("""
Python combines remarkable power with very clear syntax. It has modules, classes, exceptions, very high level
dynamic data types, and dynamic typing. There are interfaces to many system calls and libraries, as well as to
various windowing systems (X11, Motif, Tk, Mac, MFC). New built-in modules are easily written in C or C++.
Python is also usable as an extension language for applications that need a programmable interface.
""")


disc("""
Python is as old as Java and has been growing steadily in popularity for years; since our
library first came out it has entered the mainstream.  Many ReportLab library users are
already Python devotees, but if you are not, we feel that the language is an excellent
choice for document-generation apps because of its expressiveness and ability to get
data from anywhere.
""")

disc("""
Python is copyrighted but <b>freely usable and distributable, even for commercial use</b>.
""")

heading2("Acknowledgements")
disc("""Many people have contributed to ReportLab.  We would like to thank in particular 
(in alphabetical order): 
<nobr>Albertas Agejevas, 
Alex Buck, 
Andre Reitz, 
Andrew Cutler,
Andrew Mercer, 
Ben Echols,
Benjamin Dumke,
Benn B,
Chad Miller, 
Chris Buergi,
Chris Lee, 
Christian Jacobs, 
Dinu Gherman,
Edward Greve,
Eric Johnson,
Felix Labrecque,  
Fubu @ bitbucket,
Gary Poster, 
Germán M. Bravo,
Guillaume Francois, 
Hans Brand,
Henning Vonbargen,
Hosam Aly,
Ian Stevens, 
James Martin-Collar, 
Jeff Bauer,
Jerome Alet,
Jerry Casiano,
Jorge Godoy,
Keven D Smith,
Kyle MacFarlane,
Magnus Lie Hetland,
Marcel Tromp, 
Marius Gedminas,
Mark de Wit,
Matthew Duggan,
Matthias Kirst,
Matthias Klose,
Max M, 
Michael Egorov,
Michael Spector,
Mike Folwell,
Mirko Dziadzka,
Moshe Wagner,
Nate Silva,
Paul McNett, 
Peter Johnson, 
PJACock,
Publio da Costa Melo,  
Randolph Bentson,
Robert Alsina,
Robert Hölzl,
Robert Kern,
Ron Peleg,
Ruby Yocum,
Simon King,
Stephan Richter,
Steve Halasz, 
Stoneleaf @ bitbucket,
T Blatter,
Tim Roberts,
Tomasz Swiderski,
Ty Sarna,
Volker Haas,
Yoann Roman,</nobr>
and many more.""")


disc("""Special thanks go to Just van Rossum for his valuable assistance with
font technicalities.""")

disc("""Moshe Wagner and Hosam Aly deserve a huge thanks for contributing to the RTL patch, which is not yet on the trunk.""")

disc("""Marius Gedminas deserves a big hand for contributing the work on TrueType fonts and we
are glad to include these in the toolkit. Finally we thank Michal Kosmulski for the DarkGarden font
for and Bitstream Inc. for the Vera fonts.""")

heading2("Installation and Setup")

disc("""To avoid duplication, the installation instructions are kept in the README file
in our distribution, which can be viewed online at ^https://hg.reportlab.com/hg-public/reportlab/^""")

disc("""This release (%s) of ReportLab requires Python versions %s.%s+ or higher.  
	If you need to use Python 2, please use the latest ReportLab 2.7 package that is suitable for you.
""" % ((reportlab.Version,)+reportlab.__min_python_version__))



heading2("Getting Involved")
disc("""ReportLab is an Open Source project.  Although we are
a commercial company we provide the core PDF generation
sources freely, even for commercial purposes, and we make no income directly
from these modules.  We also welcome help from the community
as much as any other Open Source project.  There are many
ways in which you can help:""")

bullet("""General feedback on the core API. Does it work for you?
Are there any rough edges?  Does anything feel clunky and awkward?""")

bullet("""New objects to put in reports, or useful utilities for the library.
We have an open standard for report objects, so if you have written a nice
chart or table class, why not contribute it?""")

bullet("""Snippets and Case Studies: If you have produced some nice
output, register online on ^http://www.reportlab.com^ and submit a snippet
of your output (with or without scripts).  If ReportLab solved a
problem for you at work, write a little 'case study' and submit it.
And if your web site uses our tools to make reports, let us link to it.
We will be happy to display your work (and credit it with your name
and company) on our site!""")

bullet("""Working on the core code:  we have a long list of things
to refine or to implement.  If you are missing some features or
just want to help out, let us know!""")

disc("""The first step for anyone wanting to learn more or
get involved is to join the mailing list.  To Subscribe visit
$http://two.pairlist.net/mailman/listinfo/reportlab-users$.
From there you can also browse through the group's archives
and contributions.  The mailing list is
the place to report bugs and get support. """)

disc("""The code now lives on our website ($http://hg.reportlab.com/hg-public/reportlab/$)
in a Mercurial repository, along with an issue tracker and wiki.  Everyone should
feel free to contribute, but if you are working actively on some improvements
or want to draw attention to an issue, please use the mailing list to let us know.""")



heading2("Site Configuration")
disc("""There are a number of options which most likely need to be configured globally for a site.
The python script module $reportlab/rl_config.py$ aggregates the various settings files. You may want inspect the file $reportlab/$rl_settings.py$ which
contains defaults for the currently used variables. There are several overrides for $rl_settings"
modules $reportlab.local_rl_settings$, $reportlab_settings$ (a script file anywhere on the python path)
and finally the file $~/.reportlab_settings$ (note no .py). Temporary changes can be made using evironment variables which
are the variables from $rl_settings.py" prefixed with $RL_$ eg $RL_verbose=1$.
""")
heading3("Useful rl_config variables")
bullet("""verbose: set to integer values to control diagnostic output.""")
bullet("""shapeChecking: set this to zero to turn off a lot of error checking in the graphics modules""")
bullet("""defaultEncoding: set this to WinAnsiEncoding or MacRomanEncoding.""")
bullet("""defaultPageSize: set this to one of the values defined in reportlab/lib/pagesizes.py; as delivered
it is set to pagesizes.A4; other values are pagesizes.letter etc.""")
bullet("""defaultImageCaching: set to zero to inhibit the creation of .a85 files on your
hard-drive. The default is to create these preprocessed PDF compatible image files for faster loading""")
bullet("""T1SearchPath: this is a python list of strings representing directories that
may be queried for information on Type 1 fonts""")
bullet("""TTFSearchPath: this is a python list of strings representing directories that
may be queried for information on TrueType fonts""")
bullet("""CMapSearchPath: this is a python list of strings representing directories that
may be queried for information on font code maps.""")
bullet("""showBoundary: set to non-zero to get boundary lines drawn.""")
bullet("""ZLIB_WARNINGS: set to non-zero to get warnings if the Python compression extension is not found.""")
bullet("""pageCompression: set to non-zero to try and get compressed PDF.""")
bullet("""allowtableBoundsErrors: set to 0 to force an error on very large Platypus table elements""")
bullet("""emptyTableAction: Controls behaviour for empty tables, can be 'error' (default), 'indicate' or 'ignore'.""")
bullet("""trustedHosts: if not $None$ a list of glob patterns of trusted hosts; these may be used in places like &lt;img&gt; tags in paragraph texts.""")
bullet("""trustedSchemes: a list of allowed $URL$ schemes used with $trustedHosts$""")
disc("""For the full list of variables see the file $reportlab/rl_settings.py$.""")
heading3("Other modifications")
disc("""More complex modifications to the reportlab toolkit environment may be made using one
of the modules $rep[ortlab.local_rl_mods$ (.py script in reportlab folder),
$reportlab_mods$ (.py file on the python path) or $~/.reportlab_mods$ (note no .py).""")

heading2("Learning More About Python")

disc("""
If you are a total beginner to Python, you should check out one or more from the
growing number of resources on Python programming. The following are freely
available on the web:
""")


bullet("""<b>Python Documentation.  </b>
A list of documentation on the Python.org web site.
$http://www.python.org/doc/$
""")


bullet("""<b>Python Tutorial.  </b>
The official Python Tutorial , originally written by Guido van Rossum himself.
$http://docs.python.org/tutorial/$
""")


bullet("""<b>Learning to Program.  </b>
A tutorial on programming by Alan Gauld. Has a heavy emphasis on
Python, but also uses other languages.
$http://www.freenetpages.co.uk/hp/alan.gauld/$
""")


bullet("""<b>Instant Python</b>.
A 6-page minimal crash course by Magnus Lie Hetland.
$http://www.hetland.org/python/instant-python.php$
""")


bullet("""<b>Dive Into Python</b>.
A free Python tutorial for experienced programmers.
$http://www.diveintopython.net/$
""")


from reportlab.lib.codecharts import SingleByteEncodingChart
from tools.docco.stylesheet import getStyleSheet
styles = getStyleSheet()
indent0_style = styles['Indent0']
indent1_style = styles['Indent1']

heading2("Goals of the 3.x release series")
disc("""ReportLab 3.0 has been produced to help in the migration to Python 3.x.  Python 3.x will
be standard in future Ubuntu releases and is gaining popularity, and a good proportion
of major Python packages now run on Python 3.  """)


bullet("""Python 3.x compatibility.  A single line of code should run on 3.6 and higher""")
bullet(""" __init__.py restricts to >=3.6""")
bullet("""__init__.py allow the import of on optional reportlab.local_rl_mods to allow monkey patching etc.""")
bullet("""rl_config now imports rl_settings, optionally local_rl_settings, reportlab_settings.py & finally ~/.reportlab_settings""")
bullet("""ReportLab C extensions now live inside reportlab; _rl_accel is no longer required. All _rl_accel imports now pass through reportlab.lib.rl_accel""")
bullet("""xmllib is gone, alongside the paraparser stuff that caused issues in favour of HTMLParser.""")
bullet("""some obsolete C extensions (sgmlop and pyHnj) are gone""")
bullet("""Improved support for multi-threaded systems to the _rl_accel C extension module.""")
bullet("""Removed reportlab/lib/ para.py & pycanvas.py.  These would better belong in third party packages, which can make use of the monkeypatching feature above.""")
bullet("""Add ability to output greyscale and 1-bit PIL images without conversion to RGB. (contributed by Matthew Duggan)""")
bullet("""highlight annotation (contributed by Ben Echols)""")
bullet("""full compliance with pip, easy_install, wheels etc""")




disc("""Detailed release notes are available at 
$http://www.reportlab.com/software/documentation/relnotes/30/$""")






#Autogenerated by ReportLab guiedit do not edit
from reportlab.graphics.shapes import _DrawingEditorMixin, Drawing, Group, Polygon, PolyLine, Line, String
from reportlab.lib.colors import Color, CMYKColor, PCMYKColor

class ExplodedDrawing_Drawing(_DrawingEditorMixin,Drawing):
	def __init__(self,width=400,height=400,*args,**kw):
		Drawing.__init__(self,width,height,*args,**kw)
		self.transform = (1,0,0,1,0,0)
		self.add(Polygon(points=[108.3032,252.9412,200,288.2353,291.6968,252.9412,306.9796,138.2353,200,58.82353,93.02039,138.2353,108.3032,252.9412],fillColor=Color(1,.972549,.862745,1),fillOpacity=None,strokeColor=None,strokeWidth=0,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Polygon(points=[85.37899,266.1765,200,252.9412,261.1312,235.2941,276.414,155.8824,200,94.11765,131.2274,160.2941,85.37899,266.1765],fillColor=Color(0,1,1,1),fillOpacity=None,strokeColor=None,strokeWidth=0,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Polygon(points=[138.8688,235.2941,200,261.7647,261.1312,235.2941,329.9038,125,200,164.7059,108.3032,147.0588,138.8688,235.2941],fillColor=Color(.596078,.984314,.596078,1),fillOpacity=None,strokeColor=None,strokeWidth=0,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(PolyLine(points=[108.3032,252.9412,200,288.2353,291.6968,252.9412,306.9796,138.2353,200,58.82353,93.02039,138.2353,108.3032,252.9412],strokeColor=Color(1,.972549,.862745,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(PolyLine(points=[85.37899,266.1765,200,252.9412,261.1312,235.2941,276.414,155.8824,200,94.11765,131.2274,160.2941,85.37899,266.1765],strokeColor=Color(0,1,1,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(PolyLine(points=[138.8688,235.2941,200,261.7647,261.1312,235.2941,329.9038,125,200,164.7059,108.3032,147.0588,138.8688,235.2941],strokeColor=Color(.596078,.984314,.596078,1),strokeWidth=1,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=None,strokeOpacity=None))
		self.add(Line(200,200,200,350,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=(2,2),strokeOpacity=None))
		self.add(Line(200,200,329.9038,275,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=(2,2),strokeOpacity=None))
		self.add(Line(200,200,329.9038,125,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=(2,2),strokeOpacity=None))
		self.add(Line(200,200,200,50,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=(2,2),strokeOpacity=None))
		self.add(Line(200,200,70.09619,125,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=(2,2),strokeOpacity=None))
		self.add(Line(200,200,70.09619,275,strokeColor=Color(0,0,0,1),strokeWidth=.5,strokeLineCap=0,strokeLineJoin=0,strokeMiterLimit=0,strokeDashArray=(2,2),strokeOpacity=None))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,200,357.5)
		v0.add(String(-2.22,-4,'a',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,336.399,278.75)
		v0.add(String(-2.5,-4,'b',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,336.399,121.25)
		v0.add(String(-2.22,-4,'c',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,200,42.5)
		v0.add(String(-2.5,-4,'d',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,63.601,121.25)
		v0.add(String(-2.22,-4,'e',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))
		v0=self._nn(Group())
		v0.transform = (1,0,0,1,63.601,278.75)
		v0.add(String(-1.665,-4,'f',textAnchor='start',fontName='Times-Roman',fontSize=10,fillColor=Color(0,0,0,1)))


if __name__=="__main__": #NORUNTESTS
	ExplodedDrawing_Drawing().save(formats=['pdf'],outDir='.',fnRoot=None)



from reportlab.lib import styles
from reportlab.lib import colors
from reportlab.lib.units import cm
from reportlab.lib.enums import TA_LEFT, TA_CENTER, TA_RIGHT, TA_JUSTIFY
from reportlab.platypus import Preformatted, Paragraph, Frame, \
     Image, Table, TableStyle, Spacer
from reportlab.pdfbase import pdfmetrics
from reportlab.pdfbase.ttfonts import TTFont


def getParagraphStyles():
    """Returns a dictionary of styles to get you started.

    We will provide a way to specify a module of these.  Note that
    this just includes TableStyles as well as ParagraphStyles for any
    tables you wish to use.
    """

    pdfmetrics.registerFont(TTFont('Verdana','verdana.ttf'))
    pdfmetrics.registerFont(TTFont('Verdana-Bold','verdanab.ttf'))
    pdfmetrics.registerFont(TTFont('Verdana-Italic','verdanai.ttf'))
    pdfmetrics.registerFont(TTFont('Verdana-BoldItalic','verdanaz.ttf'))
    pdfmetrics.registerFont(TTFont('Arial Narrow','arialn.ttf'))
    pdfmetrics.registerFont(TTFont('Arial Narrow-Bold','arialnb.ttf'))
    pdfmetrics.registerFont(TTFont('Arial Narrow-Italic','arialni.ttf'))
    pdfmetrics.registerFont(TTFont('Arial Narrow-BoldItalic','arialnbi.ttf'))

    stylesheet = {}
    ParagraphStyle = styles.ParagraphStyle

    para = ParagraphStyle('Normal', None)   #the ancestor of all
    para.fontName = 'Verdana'
    para.fontSize = 28
    para.leading = 32
    para.spaceAfter = 6
    stylesheet['Normal'] = para

    #This one is spaced out a bit...
    para = ParagraphStyle('BodyText', stylesheet['Normal'])
    para.spaceBefore = 12
    stylesheet['BodyText'] = para

    #Indented, for lists
    para = ParagraphStyle('Indent', stylesheet['Normal'])
    para.leftIndent = 60
    para.firstLineIndent = 0
    stylesheet['Indent'] = para

    para = ParagraphStyle('Centered', stylesheet['Normal'])
    para.alignment = TA_CENTER
    stylesheet['Centered'] = para

    para = ParagraphStyle('BigCentered', stylesheet['Normal'])
    para.fontSize = 32
    para.alignment = TA_CENTER
    para.spaceBefore = 12
    para.spaceAfter = 12
    stylesheet['BigCentered'] = para

    para = ParagraphStyle('Italic', stylesheet['BodyText'])
    para.fontName = 'Verdana-Italic'
    stylesheet['Italic'] = para

    para = ParagraphStyle('Title', stylesheet['Normal'])
    para.fontName = 'Arial Narrow-Bold'
    para.fontSize = 48
    para.leading = 58
    para.alignment = TA_CENTER
    stylesheet['Title'] = para

    para = ParagraphStyle('Heading1', stylesheet['Normal'])
    para.fontName = 'Arial Narrow-Bold'
    para.fontSize = 40
    para.leading = 44
    para.alignment = TA_CENTER
    stylesheet['Heading1'] = para

    para = ParagraphStyle('Heading2', stylesheet['Normal'])
    para.fontName = 'Verdana'
    para.fontSize = 32
    para.leading = 36
    para.spaceBefore = 32
    para.spaceAfter = 12
    stylesheet['Heading2'] = para

    para = ParagraphStyle('Heading3', stylesheet['Normal'])
    para.fontName = 'Verdana'
    para.spaceBefore = 20
    para.spaceAfter = 6
    stylesheet['Heading3'] = para

    para = ParagraphStyle('Heading4', stylesheet['Normal'])
    para.fontName = 'Verdana-BoldItalic'
    para.spaceBefore = 6
    stylesheet['Heading4'] = para

    para = ParagraphStyle('Bullet', stylesheet['Normal'])
    para.firstLineIndent = 0
    para.leftIndent = 56
    para.spaceBefore = 6
    para.bulletFontName = 'Symbol'
    para.bulletFontSize = 24
    para.bulletIndent = 20
    stylesheet['Bullet'] = para

    para = ParagraphStyle('Bullet2', stylesheet['Normal'])
    para.firstLineIndent = 0
    para.leftIndent = 80
    para.spaceBefore = 6
    para.fontSize = 24
    para.bulletFontName = 'Symbol'
    para.bulletFontSize = 20
    para.bulletIndent = 60
    stylesheet['Bullet2'] = para

    para = ParagraphStyle('Definition', stylesheet['Normal'])
    #use this for definition lists
    para.firstLineIndent = 0
    para.leftIndent = 60
    para.bulletIndent = 0
    para.bulletFontName = 'Verdana-BoldItalic'
    para.bulletFontSize = 24
    stylesheet['Definition'] = para

    para = ParagraphStyle('Code', stylesheet['Normal'])
    para.fontName = 'Courier'
    para.fontSize = 16
    para.leading = 18
    para.leftIndent = 36
    stylesheet['Code'] = para

    para = ParagraphStyle('PythonCode', stylesheet['Normal'])
    para.fontName = 'Courier'
    para.fontSize = 16
    para.leading = 18
    para.leftIndent = 36
    stylesheet['Code'] = para

    para = ParagraphStyle('Small', stylesheet['Normal'])
    para.fontSize = 12
    para.leading = 14
    stylesheet['Small'] = para

    #now for a table
    ts = TableStyle([
         ('FONT', (0,0), (-1,-1), 'Arial Narrow', 22),
         ('LINEABOVE', (0,1), (-1,1), 2, colors.green),
         ('LINEABOVE', (0,2), (-1,-1), 0.25, colors.black),
         ('LINEBELOW', (0,-1), (-1,-1), 2, colors.green),
         ('LINEBEFORE', (0,1), (-1,-1), 2, colors.black),
         ('LINEAFTER', (0,1), (-1,-1), 2, colors.black),
         ('ALIGN', (4,1), (-1,-1), 'RIGHT'),   #all numeric cells right aligned
         ('TEXTCOLOR', (0,2), (0,-1), colors.black),
         ('BACKGROUND', (0,1), (-1,1), colors.Color(0,0.7,0.7))
         ])
    stylesheet['table1'] = ts

    return stylesheet
